@@ -39,7 +39,7 @@
  * on a uniform spatial grid.
  *
  * This program solves the problem with the DIRK method, using a
- * Newton iteration with the ARKBAND band linear solver, and a
+ * Newton iteration with the band linear solver, and a
  * user-supplied Jacobian routine.  This example uses the OpenMP 
  * vector kernel, and employs OpenMP threading within the 
  * right-hand side and Jacobian construction functions.
@@ -53,41 +53,52 @@
 #include <stdlib.h>
 #include <math.h>
 #include <arkode/arkode.h>            /* prototypes for ARKode fcts., consts. */
-#include <nvector/nvector_openmp.h>   /* OpenMP N_Vector types, fcts., macros */
-#include <arkode/arkode_band.h>       /* prototype for ARKBand solver */
+#include <nvector/nvector_openmp.h>   /* access to OpenMP N_Vector */
+#include <sunmatrix/sunmatrix_band.h> /* access to band SUNMatrix */
+#include <sunlinsol/sunlinsol_band.h> /* access to band SUNLinearSolver */
+#include <arkode/arkode_direct.h>     /* access to ARKDls interface */
 #include <sundials/sundials_types.h>  /* def. of type 'realtype' */
 #include <sundials/sundials_math.h>   /* def. of SUNRsqrt, etc. */
 #ifdef _OPENMP
 #include <omp.h>                      /* OpenMP functions */
 #endif
 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+#define GSYM "Lg"
+#define ESYM "Le"
+#define FSYM "Lf"
+#else
+#define GSYM "g"
+#define ESYM "e"
+#define FSYM "f"
+#endif
+
 /* accessor macros between (x,v) location and 1D NVector array */
 #define IDX(x,v) (3*(x)+v)
 
 /* user data structure */
 typedef struct {  
-  long int N;    /* number of intervals      */
-  int nthreads;  /* number of OpenMP threads */
-  realtype dx;   /* mesh spacing             */
-  realtype a;    /* constant forcing on u    */
-  realtype b;    /* steady-state value of w  */
-  realtype du;   /* diffusion coeff for u    */
-  realtype dv;   /* diffusion coeff for v    */
-  realtype dw;   /* diffusion coeff for w    */
-  realtype ep;   /* stiffness parameter      */
+  sunindextype N;  /* number of intervals      */
+  int nthreads;    /* number of OpenMP threads */
+  realtype dx;     /* mesh spacing             */
+  realtype a;      /* constant forcing on u    */
+  realtype b;      /* steady-state value of w  */
+  realtype du;     /* diffusion coeff for u    */
+  realtype dv;     /* diffusion coeff for v    */
+  realtype dw;     /* diffusion coeff for w    */
+  realtype ep;     /* stiffness parameter      */
 } *UserData;
 
 
 /* User-supplied Functions Called by the Solver */
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data);
-static int Jac(long int N, long int mu, long int ml,
-               realtype t, N_Vector y, N_Vector fy, 
-               DlsMat J, void *user_data,
+static int Jac(realtype t, N_Vector y, N_Vector fy, 
+               SUNMatrix J, void *user_data,
                N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
 
 /* Private helper functions  */
-static int LaplaceMatrix(realtype c, DlsMat Jac, UserData udata);
-static int ReactionJac(realtype c, N_Vector y, DlsMat Jac, UserData udata);
+static int LaplaceMatrix(realtype c, SUNMatrix Jac, UserData udata);
+static int ReactionJac(realtype c, N_Vector y, SUNMatrix Jac, UserData udata);
 
 /* Private function to check function return values */
 static int check_flag(void *flagvalue, const char *funcname, int opt);
@@ -102,7 +113,7 @@ int main(int argc, char *argv[])
   int Nvar = 3;                 /* number of solution fields */
   UserData udata = NULL;
   realtype *data;
-  long int N = 201;             /* spatial mesh size */
+  sunindextype N = 201;         /* spatial mesh size */
   realtype a = 0.6;             /* problem parameters */
   realtype b = 2.0;
   realtype du = 0.025;
@@ -111,7 +122,7 @@ int main(int argc, char *argv[])
   realtype ep = 1.0e-5;         /* stiffness parameter */
   realtype reltol = 1.0e-6;     /* tolerances */
   realtype abstol = 1.0e-10;
-  long int NEQ, i;
+  sunindextype NEQ, i;
 
   /* general problem variables */
   int flag;                     /* reusable error-checking flag */
@@ -119,6 +130,8 @@ int main(int argc, char *argv[])
   N_Vector umask = NULL;        /* empty mask vectors for viewing solution components */
   N_Vector vmask = NULL;
   N_Vector wmask = NULL;
+  SUNMatrix A = NULL;           /* empty matrix for linear solver */
+  SUNLinearSolver LS = NULL;    /* empty linear solver structure */
   void *arkode_mem = NULL;      /* empty ARKode memory structure */
   realtype pi, t, dTout, tout, u, v, w;
   FILE *FID, *UFID, *VFID, *WFID;
@@ -130,11 +143,11 @@ int main(int argc, char *argv[])
   if (check_flag((void *) udata, "malloc", 2)) return 1;
 
   /* set the number of threads to use */
-  num_threads = 1; /* default value */
+  num_threads = 1;                       /* default value */
 #ifdef _OPENMP
   num_threads = omp_get_max_threads();   /* overwrite with OMP_NUM_THREADS environment variable */
 #endif
-  if (argc > 1)   /* overwrite with command line value, if supplied */
+  if (argc > 1)                          /* overwrite with command line value, if supplied */
     num_threads = strtol(argv[1], NULL, 0);
 
   /* store the inputs in the UserData structure */
@@ -152,15 +165,15 @@ int main(int argc, char *argv[])
 
   /* Initial problem output */
   printf("\n1D Brusselator PDE test problem:\n");
-  printf("    N = %li,  NEQ = %li\n", udata->N, NEQ);
+  printf("    N = %li,  NEQ = %li\n", (long int) udata->N, (long int) NEQ);
   printf("    num_threads = %i\n", num_threads);
-  printf("    problem parameters:  a = %g,  b = %g,  ep = %g\n",
+  printf("    problem parameters:  a = %"GSYM",  b = %"GSYM",  ep = %"GSYM"\n",
 	 udata->a, udata->b, udata->ep);
-  printf("    diffusion coefficients:  du = %g,  dv = %g,  dw = %g\n", 
+  printf("    diffusion coefficients:  du = %"GSYM",  dv = %"GSYM",  dw = %"GSYM"\n", 
 	 udata->du, udata->dv, udata->dw);
-  printf("    reltol = %.1e,  abstol = %.1e\n\n", reltol, abstol);
+  printf("    reltol = %.1"ESYM",  abstol = %.1"ESYM"\n\n", reltol, abstol);
 
-  /* Initialize data structures */
+  /* Initialize vector data structures */
   y = N_VNew_OpenMP(NEQ, num_threads);      /* Create vector for solution */
   if (check_flag((void *)y, "N_VNew_OpenMP", 0)) return 1;
   udata->dx = RCONST(1.0)/(N-1);            /* set spatial mesh spacing */
@@ -197,6 +210,12 @@ int main(int argc, char *argv[])
   if (check_flag((void *) data, "N_VGetArrayPointer", 0)) return 1;
   for (i=0; i<N; i++)  data[IDX(i,2)] = RCONST(1.0);
 
+  /* Initialize matrix and linear solver data structures */
+  A = SUNBandMatrix(NEQ, 4, 4, 8);
+  if (check_flag((void *)A, "SUNBandMatrix", 0)) return 1;
+  LS = SUNBandLinearSolver(y, A);
+  if (check_flag((void *)LS, "SUNBandLinearSolver", 0)) return 1;
+  
   /* Create the solver memory */
   arkode_mem = ARKodeCreate();
   if (check_flag((void *)arkode_mem, "ARKodeCreate", 0)) return 1;
@@ -215,14 +234,14 @@ int main(int argc, char *argv[])
   if (check_flag(&flag, "ARKodeSStolerances", 1)) return 1;
 
   /* Linear solver specification */
-  flag = ARKBand(arkode_mem, NEQ, 4, 4);          /* Specify the band linear solver */
-  if (check_flag(&flag, "ARKBand", 1)) return 1;
-  flag = ARKDlsSetBandJacFn(arkode_mem, Jac);     /* Set the Jacobian routine */
-  if (check_flag(&flag, "ARKDlsSetBandJacFn", 1)) return 1;
+  flag = ARKDlsSetLinearSolver(arkode_mem, LS, A);          /* Attach matrix and linear solver */
+  if (check_flag(&flag, "ARKDlsSetLinearSolver", 1)) return 1;
+  flag = ARKDlsSetJacFn(arkode_mem, Jac);                   /* Set the Jacobian routine */
+  if (check_flag(&flag, "ARKDlsSetJacFn", 1)) return 1;
 
   /* output spatial mesh to disk */
   FID=fopen("bruss_mesh.txt","w");
-  for (i=0; i<N; i++)  fprintf(FID,"  %.16e\n", udata->dx*i);
+  for (i=0; i<N; i++)  fprintf(FID,"  %.16"ESYM"\n", udata->dx*i);
   fclose(FID);
   
   /* Open output stream for results, access data arrays */
@@ -233,9 +252,9 @@ int main(int argc, char *argv[])
   /* output initial condition to disk */
   data = N_VGetArrayPointer(y);
   if (check_flag((void *)data, "N_VGetArrayPointer", 0)) return 1;
-  for (i=0; i<N; i++)  fprintf(UFID," %.16e", data[IDX(i,0)]);
-  for (i=0; i<N; i++)  fprintf(VFID," %.16e", data[IDX(i,1)]);
-  for (i=0; i<N; i++)  fprintf(WFID," %.16e", data[IDX(i,2)]);
+  for (i=0; i<N; i++)  fprintf(UFID," %.16"ESYM, data[IDX(i,0)]);
+  for (i=0; i<N; i++)  fprintf(VFID," %.16"ESYM, data[IDX(i,1)]);
+  for (i=0; i<N; i++)  fprintf(WFID," %.16"ESYM, data[IDX(i,2)]);
   fprintf(UFID,"\n");
   fprintf(VFID,"\n");
   fprintf(WFID,"\n");
@@ -257,7 +276,7 @@ int main(int argc, char *argv[])
     v = SUNRsqrt(v*v/N);
     w = N_VWL2Norm(y,wmask);
     w = SUNRsqrt(w*w/N);
-    printf("  %10.6f  %10.6f  %10.6f  %10.6f\n", t, u, v, w);
+    printf("  %10.6"FSYM"  %10.6"FSYM"  %10.6"FSYM"  %10.6"FSYM"\n", t, u, v, w);
     if (flag >= 0) {                                       /* successful solve: update output time */
       tout += dTout;
       tout = (tout > Tf) ? Tf : tout;
@@ -267,9 +286,9 @@ int main(int argc, char *argv[])
     }
 
     /* output results to disk */
-    for (i=0; i<N; i++)  fprintf(UFID," %.16e", data[IDX(i,0)]);
-    for (i=0; i<N; i++)  fprintf(VFID," %.16e", data[IDX(i,1)]);
-    for (i=0; i<N; i++)  fprintf(WFID," %.16e", data[IDX(i,2)]);
+    for (i=0; i<N; i++)  fprintf(UFID," %.16"ESYM, data[IDX(i,0)]);
+    for (i=0; i<N; i++)  fprintf(VFID," %.16"ESYM, data[IDX(i,1)]);
+    for (i=0; i<N; i++)  fprintf(WFID," %.16"ESYM, data[IDX(i,2)]);
     fprintf(UFID,"\n");
     fprintf(VFID,"\n");
     fprintf(WFID,"\n");
@@ -310,12 +329,14 @@ int main(int argc, char *argv[])
   printf("   Total number of error test failures = %li\n\n", netf);
 
   /* Clean up and return with successful completion */
+  free(udata);                  /* Free user data */
+  ARKodeFree(&arkode_mem);      /* Free integrator memory */
+  SUNLinSolFree(LS);            /* Free linear solver */
+  SUNMatDestroy(A);             /* Free matrix */
   N_VDestroy_OpenMP(y);         /* Free vectors */
   N_VDestroy_OpenMP(umask);
   N_VDestroy_OpenMP(vmask);
   N_VDestroy_OpenMP(wmask);
-  free(udata);                  /* Free user data */
-  ARKodeFree(&arkode_mem);      /* Free integrator memory */
   return 0;
 }
 
@@ -327,7 +348,7 @@ int main(int argc, char *argv[])
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
 {
   UserData udata = (UserData) user_data;      /* access problem data */
-  long int N  = udata->N;                     /* set variable shortcuts */
+  sunindextype N = udata->N;                  /* set variable shortcuts */
   realtype a  = udata->a;
   realtype b  = udata->b;
   realtype ep = udata->ep;
@@ -337,7 +358,7 @@ static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
   realtype dx = udata->dx;
   realtype *Ydata=NULL, *dYdata=NULL;
   realtype uconst, vconst, wconst, u, ul, ur, v, vl, vr, w, wl, wr;
-  long int i;
+  sunindextype i;
 
   /* clear out ydot (to be careful) */
   N_VConst(0.0, ydot);
@@ -380,13 +401,12 @@ static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
 
 
 /* Jacobian routine to compute J(t,y) = df/dy. */
-static int Jac(long int M, long int mu, long int ml,
-               realtype t, N_Vector y, N_Vector fy, 
-               DlsMat J, void *user_data,
+static int Jac(realtype t, N_Vector y, N_Vector fy, 
+               SUNMatrix J, void *user_data,
                N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
-  UserData udata = (UserData) user_data;     /* access problem data */
-  SetToZero(J);                              /* Initialize Jacobian to zero */
+  UserData udata = (UserData) user_data;  /* access problem data */
+  SUNMatZero(J);                          /* Initialize Jacobian to zero */
 
   /* Fill in the Laplace matrix */
   if (LaplaceMatrix(RCONST(1.0), J, udata)) {
@@ -409,26 +429,29 @@ static int Jac(long int M, long int mu, long int ml,
 
 /* Routine to compute the stiffness matrix from (L*y), scaled by the factor c.
    We add the result into Jac and do not erase what was already there */
-static int LaplaceMatrix(realtype c, DlsMat Jac, UserData udata)
+static int LaplaceMatrix(realtype c, SUNMatrix Jac, UserData udata)
 {
-  long int i;                /* set shortcuts */
-  long int N = udata->N;
+  sunindextype N = udata->N;            /* set shortcuts */
   realtype dx = udata->dx;
+  sunindextype i;
+  realtype uconst = c*udata->du/dx/dx;
+  realtype vconst = c*udata->dv/dx/dx;
+  realtype wconst = c*udata->dw/dx/dx;
   
   /* iterate over intervals, filling in Jacobian entries */
 #pragma omp parallel for default(shared) private(i) schedule(static) num_threads(udata->nthreads)
   for (i=1; i<N-1; i++) {
 
     /* Jacobian of (L*y) at this node */
-    BAND_ELEM(Jac,IDX(i,0),IDX(i-1,0)) += c*udata->du/dx/dx;
-    BAND_ELEM(Jac,IDX(i,1),IDX(i-1,1)) += c*udata->dv/dx/dx;
-    BAND_ELEM(Jac,IDX(i,2),IDX(i-1,2)) += c*udata->dw/dx/dx;
-    BAND_ELEM(Jac,IDX(i,0),IDX(i,0)) += -c*RCONST(2.0)*udata->du/dx/dx;
-    BAND_ELEM(Jac,IDX(i,1),IDX(i,1)) += -c*RCONST(2.0)*udata->dv/dx/dx;
-    BAND_ELEM(Jac,IDX(i,2),IDX(i,2)) += -c*RCONST(2.0)*udata->dw/dx/dx;
-    BAND_ELEM(Jac,IDX(i,0),IDX(i+1,0)) += c*udata->du/dx/dx;
-    BAND_ELEM(Jac,IDX(i,1),IDX(i+1,1)) += c*udata->dv/dx/dx;
-    BAND_ELEM(Jac,IDX(i,2),IDX(i+1,2)) += c*udata->dw/dx/dx;
+    SM_ELEMENT_B(Jac,IDX(i,0),IDX(i-1,0)) += uconst;
+    SM_ELEMENT_B(Jac,IDX(i,1),IDX(i-1,1)) += vconst;
+    SM_ELEMENT_B(Jac,IDX(i,2),IDX(i-1,2)) += wconst;
+    SM_ELEMENT_B(Jac,IDX(i,0),IDX(i,0)) -= RCONST(2.0)*uconst;
+    SM_ELEMENT_B(Jac,IDX(i,1),IDX(i,1)) -= RCONST(2.0)*vconst;
+    SM_ELEMENT_B(Jac,IDX(i,2),IDX(i,2)) -= RCONST(2.0)*wconst;
+    SM_ELEMENT_B(Jac,IDX(i,0),IDX(i+1,0)) += uconst;
+    SM_ELEMENT_B(Jac,IDX(i,1),IDX(i+1,1)) += vconst;
+    SM_ELEMENT_B(Jac,IDX(i,2),IDX(i+1,2)) += wconst;
   }
 
   return 0;
@@ -438,12 +461,12 @@ static int LaplaceMatrix(realtype c, DlsMat Jac, UserData udata)
 
 /* Routine to compute the Jacobian matrix from R(y), scaled by the factor c.
    We add the result into Jac and do not erase what was already there */
-static int ReactionJac(realtype c, N_Vector y, DlsMat Jac, UserData udata)
+static int ReactionJac(realtype c, N_Vector y, SUNMatrix Jac, UserData udata)
 {
-  long int N  = udata->N;                      /* set shortcuts */
-  long int i;
-  realtype u, v, w;
+  sunindextype N = udata->N;                   /* set shortcuts */
   realtype ep = udata->ep;
+  sunindextype i;
+  realtype u, v, w;
   realtype *Ydata = N_VGetArrayPointer(y);     /* access solution array */
   if (check_flag((void *)Ydata, "N_VGetArrayPointer", 0)) return 1;
   
@@ -457,18 +480,18 @@ static int ReactionJac(realtype c, N_Vector y, DlsMat Jac, UserData udata)
     w = Ydata[IDX(i,2)];
 
     /* all vars wrt u */
-    BAND_ELEM(Jac,IDX(i,0),IDX(i,0)) += c*(RCONST(2.0)*u*v-(w+RCONST(1.0)));
-    BAND_ELEM(Jac,IDX(i,1),IDX(i,0)) += c*(w - RCONST(2.0)*u*v);
-    BAND_ELEM(Jac,IDX(i,2),IDX(i,0)) += c*(-w);
+    SM_ELEMENT_B(Jac,IDX(i,0),IDX(i,0)) += c*(RCONST(2.0)*u*v-(w+RCONST(1.0)));
+    SM_ELEMENT_B(Jac,IDX(i,1),IDX(i,0)) += c*(w - RCONST(2.0)*u*v);
+    SM_ELEMENT_B(Jac,IDX(i,2),IDX(i,0)) += c*(-w);
 
     /* all vars wrt v */
-    BAND_ELEM(Jac,IDX(i,0),IDX(i,1)) += c*(u*u);
-    BAND_ELEM(Jac,IDX(i,1),IDX(i,1)) += c*(-u*u);
+    SM_ELEMENT_B(Jac,IDX(i,0),IDX(i,1)) += c*(u*u);
+    SM_ELEMENT_B(Jac,IDX(i,1),IDX(i,1)) += c*(-u*u);
 
     /* all vars wrt w */
-    BAND_ELEM(Jac,IDX(i,0),IDX(i,2)) += c*(-u);
-    BAND_ELEM(Jac,IDX(i,1),IDX(i,2)) += c*(u);
-    BAND_ELEM(Jac,IDX(i,2),IDX(i,2)) += c*(-RCONST(1.0)/ep - u);
+    SM_ELEMENT_B(Jac,IDX(i,0),IDX(i,2)) += c*(-u);
+    SM_ELEMENT_B(Jac,IDX(i,1),IDX(i,2)) += c*(u);
+    SM_ELEMENT_B(Jac,IDX(i,2),IDX(i,2)) += c*(-RCONST(1.0)/ep - u);
 
   }
 
