@@ -35,7 +35,7 @@
  * [0, 1], but as the simulation proceeds the mesh is adapted.
  *
  * This program solves the problem with a DIRK method, solved with 
- * a Newton iteration and PCG linear solver, with a user-supplied 
+ * a Newton iteration and SUNPCG linear solver, with a user-supplied 
  * Jacobian-vector product routine.
  *---------------------------------------------------------------*/
 
@@ -45,13 +45,24 @@
 #include <math.h>
 #include <arkode/arkode.h>            /* prototypes for ARKode fcts., consts. */
 #include <nvector/nvector_serial.h>   /* serial N_Vector types, fcts., macros */
-#include <arkode/arkode_pcg.h>        /* prototype for ARKPcg solver */
-#include <sundials/sundials_types.h>  /* def. of type 'realtype' */
-#include <sundials/sundials_math.h>   /* def. of SUNRsqrt, etc. */
+#include <sunlinsol/sunlinsol_pcg.h>  /* access to PCG SUNLinearSolver        */
+#include <arkode/arkode_spils.h>      /* access to ARKSpils interface         */
+#include <sundials/sundials_types.h>  /* defs. of realtype, sunindextype, etc */
+#include <sundials/sundials_math.h>   /* def. of SUNRsqrt, etc.               */
+
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+#define GSYM "Lg"
+#define ESYM "Le"
+#define FSYM "Lf"
+#else
+#define GSYM "g"
+#define ESYM "e"
+#define FSYM "f"
+#endif
 
 /* user data structure */
 typedef struct {
-  long int N;           /* current number of intervals */
+  sunindextype N;       /* current number of intervals */
   realtype *x;          /* current mesh */
   realtype k;           /* diffusion coefficient */
   realtype refine_tol;  /* adaptivity tolerance */
@@ -60,12 +71,12 @@ typedef struct {
 /* User-supplied Functions Called by the Solver */
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data);
 static int Jac(N_Vector v, N_Vector Jv, realtype t, N_Vector y,
-            N_Vector fy, void *user_data, N_Vector tmp);
+               N_Vector fy, void *user_data, N_Vector tmp);
 
 /* Private function to check function return values */
-realtype * adapt_mesh(N_Vector y, long int *Nnew, UserData udata);
-static int project(long int Nold, realtype *xold, N_Vector yold, 
-		   long int Nnew, realtype *xnew, N_Vector ynew);
+realtype * adapt_mesh(N_Vector y, sunindextype *Nnew, UserData udata);
+static int project(sunindextype Nold, realtype *xold, N_Vector yold, 
+                   sunindextype Nnew, realtype *xnew, N_Vector ynew);
 static int check_flag(void *flagvalue, const char *funcname, int opt);
 
 /* Main Program */
@@ -79,10 +90,11 @@ int main() {
   realtype hscale = 1.0;       /* time step change factor on resizes */
   UserData udata = NULL;
   realtype *data;
-  long int N = 21;             /* initial spatial mesh size */
+  sunindextype N = 21;             /* initial spatial mesh size */
   realtype refine = 3.e-3;     /* adaptivity refinement tolerance */
   realtype k = 0.5;            /* heat conductivity */
-  long int i, nni, nni_cur=0, nni_tot=0, nli, nli_tot=0;
+  sunindextype i;
+  long int nni, nni_cur=0, nni_tot=0, nli, nli_tot=0;
   int iout=0;
 
   /* general problem variables */
@@ -90,11 +102,12 @@ int main() {
   N_Vector y  = NULL;          /* empty vector for storing solution */
   N_Vector y2 = NULL;          /* empty vector for storing solution */
   N_Vector yt = NULL;          /* empty vector for swapping */
+  SUNLinearSolver LS = NULL;   /* empty linear solver object */
   void *arkode_mem = NULL;     /* empty ARKode memory structure */
   FILE *XFID, *UFID;
   realtype t, olddt, newdt;
   realtype *xnew = NULL;
-  long int Nnew;
+  sunindextype Nnew;
 
   /* allocate and fill initial udata structure */
   udata = (UserData) malloc(sizeof(*udata));
@@ -106,8 +119,8 @@ int main() {
 
   /* Initial problem output */
   printf("\n1D adaptive Heat PDE test problem:\n");
-  printf("  diffusion coefficient:  k = %g\n", udata->k);
-  printf("  initial N = %li\n", udata->N);
+  printf("  diffusion coefficient:  k = %"GSYM"\n", udata->k);
+  printf("  initial N = %li\n", (long int) udata->N);
 
   /* Initialize data structures */
   y = N_VNew_Serial(N);       /* Create initial serial vector for solution */
@@ -118,7 +131,7 @@ int main() {
   XFID=fopen("heat_mesh.txt","w");
 
   /* output initial mesh to disk */
-  for (i=0; i<udata->N; i++)  fprintf(XFID," %.16e", udata->x[i]);
+  for (i=0; i<udata->N; i++)  fprintf(XFID," %.16"ESYM, udata->x[i]);
   fprintf(XFID,"\n");
 
   /* Open output stream for results, access data array */
@@ -126,7 +139,7 @@ int main() {
 
   /* output initial condition to disk */
   data = N_VGetArrayPointer(y);
-  for (i=0; i<udata->N; i++)  fprintf(UFID," %.16e", data[i]);
+  for (i=0; i<udata->N; i++)  fprintf(UFID," %.16"ESYM, data[i]);
   fprintf(UFID,"\n");
 
 
@@ -150,11 +163,19 @@ int main() {
   flag = ARKodeSetPredictorMethod(arkode_mem, 0);     /* Set predictor method */
   if (check_flag(&flag, "ARKodeSetPredictorMethod", 1)) return 1;
 
-  /* Linear solver specification */
-  flag = ARKPcg(arkode_mem, 0, N);
-  if (check_flag(&flag, "ARKPcg", 1)) return 1;
-  flag = ARKSpilsSetJacTimesVecFn(arkode_mem, Jac);
-  if (check_flag(&flag, "ARKSpilsSetJacTimesVecFn", 1)) return 1;
+  /* Specify linearly implicit RHS, with time-dependent Jacobian */
+  flag = ARKodeSetLinear(arkode_mem, 1);
+  if (check_flag(&flag, "ARKodeSetLinear", 1)) return 1;
+
+  /* Initialize PCG solver -- no preconditioning, with up to N iterations  */
+  LS = SUNPCG(y, 0, N);
+  if (check_flag((void *)LS, "SUNPCG", 0)) return 1;
+  
+  /* Linear solver interface -- set user-supplied J*v routine (no 'jtsetup' required) */
+  flag = ARKSpilsSetLinearSolver(arkode_mem, LS);        /* Attach linear solver to ARKode */
+  if (check_flag(&flag, "ARKSpilsSetLinearSolver", 1)) return 1;
+  flag = ARKSpilsSetJacTimes(arkode_mem, NULL, Jac);     /* Set the Jacobian routine */
+  if (check_flag(&flag, "ARKSpilsSetJacTimes", 1)) return 1;
 
   /* Main time-stepping loop: calls ARKode to perform the integration, then
      prints results.  Stops when the final time has been reached */
@@ -163,8 +184,9 @@ int main() {
   newdt = 0.0;
   printf("  iout          dt_old                 dt_new               ||u||_rms       N   NNI  NLI\n");
   printf(" ----------------------------------------------------------------------------------------\n");
-  printf(" %4i  %19.15e  %19.15e  %19.15e  %li   %2i  %3i\n", 
-	 iout, olddt, newdt, SUNRsqrt(N_VDotProd(y,y)/udata->N), udata->N, 0, 0);
+  printf(" %4i  %19.15"ESYM"  %19.15"ESYM"  %19.15"ESYM"  %li   %2i  %3i\n", 
+	 iout, olddt, newdt, SUNRsqrt(N_VDotProd(y,y)/udata->N),
+         (long int) udata->N, 0, 0);
   while (t < Tf) {
 
     /* "set" routines */
@@ -189,17 +211,18 @@ int main() {
 
     /* print current solution stats */
     iout++;
-    printf(" %4i  %19.15e  %19.15e  %19.15e  %li   %2li  %3li\n", 
-	   iout, olddt, newdt, SUNRsqrt(N_VDotProd(y,y)/udata->N), udata->N, nni-nni_cur, nli);
+    printf(" %4i  %19.15"ESYM"  %19.15"ESYM"  %19.15"ESYM"  %li   %2li  %3li\n", 
+	   iout, olddt, newdt, SUNRsqrt(N_VDotProd(y,y)/udata->N),
+           (long int) udata->N, nni-nni_cur, nli);
     nni_cur = nni;
     nni_tot = nni;
     nli_tot += nli;
 
     /* output results and current mesh to disk */
     data = N_VGetArrayPointer(y);
-    for (i=0; i<udata->N; i++)  fprintf(UFID," %.16e", data[i]);
+    for (i=0; i<udata->N; i++)  fprintf(UFID," %.16"ESYM, data[i]);
     fprintf(UFID,"\n");
-    for (i=0; i<udata->N; i++)  fprintf(XFID," %.16e", udata->x[i]);
+    for (i=0; i<udata->N; i++)  fprintf(XFID," %.16"ESYM, udata->x[i]);
     fprintf(XFID,"\n");
 
     /* adapt the spatial mesh */
@@ -215,7 +238,7 @@ int main() {
     if (check_flag(&flag, "project", 1)) return 1;
 
     /* delete old vector, old mesh */
-    N_VDestroy_Serial(y);
+    N_VDestroy(y);
     free(udata->x);
     
     /* swap x and xnew so that new mesh is stored in udata structure */
@@ -232,18 +255,18 @@ int main() {
     flag = ARKodeResize(arkode_mem, y, hscale, t, NULL, NULL);
     if (check_flag(&flag, "ARKodeResize", 1)) return 1;
 
-    /* destroy and re-allocate linear solver memory */
-    flag = ARKPcg(arkode_mem, 0, udata->N);
-    if (check_flag(&flag, "ARKPcg", 1)) return 1;
-    flag = ARKSpilsSetJacTimesVecFn(arkode_mem, Jac);
-    if (check_flag(&flag, "ARKSpilsSetJacTimesVecFn", 1)) return 1;
+    /* destroy and re-allocate linear solver memory; reattach to ARKSpils interface */
+    SUNLinSolFree(LS);
+    LS = SUNPCG(y, 0, N);
+    if (check_flag((void *)LS, "SUNPCG", 0)) return 1;
+    flag = ARKSpilsSetLinearSolver(arkode_mem, LS);
+    if (check_flag(&flag, "ARKSpilsSetLinearSolver", 1)) return 1;
+    flag = ARKSpilsSetJacTimes(arkode_mem, NULL, Jac);
+    if (check_flag(&flag, "ARKSpilsSetJacTimes", 1)) return 1;
 
   }
   printf(" ----------------------------------------------------------------------------------------\n");
 
-  /* Free integrator memory */
-  ARKodeFree(&arkode_mem);
-
   /* print some final statistics */
   printf(" Final solver statistics:\n");
   printf("   Total number of time steps = %i\n", iout);
@@ -253,9 +276,11 @@ int main() {
   /* Clean up and return with successful completion */
   fclose(UFID);
   fclose(XFID);
-  N_VDestroy_Serial(y);        /* Free vectors */
+  N_VDestroy(y);               /* Free vectors */
   free(udata->x);              /* Free user data */
   free(udata);   
+  ARKodeFree(&arkode_mem);     /* Free integrator memory */
+  SUNLinSolFree(LS);           /* Free linear solver */
 
   return 0;
 }
@@ -268,12 +293,12 @@ int main() {
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
 {
   UserData udata = (UserData) user_data;    /* access problem data */
-  long int N  = udata->N;                   /* set variable shortcuts */
+  sunindextype N = udata->N;                /* set variable shortcuts */
   realtype k  = udata->k;
   realtype *x = udata->x;
   realtype *Y=NULL, *Ydot=NULL;
   realtype dxL, dxR;
-  long int i;
+  sunindextype i;
   Y = N_VGetArrayPointer(y);      /* access data arrays */
   if (check_flag((void *) Y, "N_VGetArrayPointer", 0)) return 1;
   Ydot = N_VGetArrayPointer(ydot);
@@ -304,15 +329,15 @@ static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
 
 /* Jacobian routine to compute J(t,y) = df/dy. */
 static int Jac(N_Vector v, N_Vector Jv, realtype t, N_Vector y, 
-	       N_Vector fy, void *user_data, N_Vector tmp)
+               N_Vector fy, void *user_data, N_Vector tmp)
 {
   UserData udata = (UserData) user_data;     /* variable shortcuts */
-  long int N  = udata->N;
+  sunindextype N = udata->N;
   realtype k  = udata->k;
   realtype *x = udata->x;
   realtype *V=NULL, *JV=NULL;
   realtype dxL, dxR;
-  long int i;
+  sunindextype i;
   V = N_VGetArrayPointer(v);       /* access data arrays */
   if (check_flag((void *) V, "N_VGetArrayPointer", 0)) return 1;
   JV = N_VGetArrayPointer(Jv);
@@ -345,12 +370,12 @@ static int Jac(N_Vector v, N_Vector Jv, realtype t, N_Vector y,
       Nnew [output] -- the size of the new mesh
       udata [input] -- the current system information 
    The return for this function is a pointer to the new mesh. */
-realtype * adapt_mesh(N_Vector y, long int *Nnew, UserData udata)
+realtype * adapt_mesh(N_Vector y, sunindextype *Nnew, UserData udata)
 {
-  int i, j;
+  sunindextype i, j;
   int *marks=NULL;
   realtype ydd, *xold=NULL, *Y=NULL, *xnew=NULL;
-  long int num_refine, N_new;
+  sunindextype num_refine, N_new;
 
   /* Access current solution and mesh arrays */
   xold = udata->x;
@@ -360,19 +385,6 @@ realtype * adapt_mesh(N_Vector y, long int *Nnew, UserData udata)
   /* create marking array */
   marks = calloc(udata->N-1, sizeof(int));
 
-  /* /\* perform marking:  */
-  /*     0 -> leave alone */
-  /*     1 -> refine */
-  /* realtype ymax, ymin; */
-  /* for (i=0; i<(udata->N-1); i++) { */
-
-  /*   /\* check for refinement *\/ */
-  /*   if (fabs(Y[i+1] - Y[i]) > udata->refine_tol) { */
-  /*     marks[i] = 1; */
-  /*     continue; */
-  /*   } */
-  /* } */
-
   /* perform marking: 
       0 -> leave alone
       1 -> refine */
@@ -440,10 +452,10 @@ realtype * adapt_mesh(N_Vector y, long int *Nnew, UserData udata)
       xnew [input] -- the new mesh
       ynew [output] -- the vector defined over the new mesh
                        (allocated prior to calling project) */
-static int project(long int Nold, realtype *xold, N_Vector yold, 
-		   long int Nnew, realtype *xnew, N_Vector ynew)
+static int project(sunindextype Nold, realtype *xold, N_Vector yold, 
+		   sunindextype Nnew, realtype *xnew, N_Vector ynew)
 {
-  int iv, i, j;
+  sunindextype iv, i, j;
   realtype *Yold=NULL, *Ynew=NULL;
 
   /* Access data arrays */
