@@ -47,8 +47,8 @@
 !    a=0.6,  b=2.0,  du=0.025,  dv=0.025,  dw=0.025,  ep=1.d-5
 !
 ! This program solves the problem with the DIRK method, using a 
-! Newton iteration with the ARKKLU and ARKMASSKLU sparse linear 
-! solvers, where we store the Jacobian and mass matrices in 
+! Newton iteration with the SUNKLU sparse linear solvers for both 
+! the system and mass matrices.  These matrices are stored in 
 ! compressed-sparse-row format.
 !
 ! Output is printed 10 times throughout the defined time interval.
@@ -59,9 +59,10 @@
 ! user data structure
 module UserData
   implicit none
+  include "sundials/sundials_fconfig.h"
   save
   
-  integer :: N                     ! number of intervals
+  integer*8 :: N                   ! number of intervals
   real*8, allocatable :: x(:)      ! mesh node locations
   real*8 :: a                      ! constant forcing on u
   real*8 :: b                      ! steady-state value of w
@@ -163,21 +164,24 @@ end module Quadrature
 ! Main driver program
 !-----------------------------------------------------------------
 program driver
-  use UserData
 
-  ! Declarations
+  ! inclusions
+  use UserData
   implicit none
 
+  ! Declarations
   ! general problem variables
-  real*8, parameter :: T0=0.d0, Tf=10.d0
-  real*8    :: dTout, Tout, Tcur, rtol, atol, pi, h, z, rout(6)
-  integer   :: i, it, Nt, ier, idef, ineq, nnz, ordering, sparsetype
-  integer*8 :: NEQ, iout(29)
-  real*8, allocatable :: y(:,:), umask(:,:), vmask(:,:), wmask(:,:)
+  real(kind=REALTYPE), parameter :: T0=0.d0, Tf=10.d0
+  real(kind=REALTYPE) :: rtol, atol, rout(6), Tout, Tcur
+  real*8    :: dTout, pi, h, z
+  integer   :: i, it, Nt, ier, ordering, sparsetype, time_dep
+  integer*8 :: iout(29), NEQ, nnz, Iinput
+  real(kind=REALTYPE), allocatable :: y(:,:), umask(:,:)
+  real(kind=REALTYPE), allocatable :: vmask(:,:), wmask(:,:)
 
   ! dummy real/integer parameters to pass through to supplied functions
   integer*8 :: ipar
-  real*8    :: rpar
+  real(kind=REALTYPE) :: rpar
 
   !-----------------------
 
@@ -239,6 +243,17 @@ program driver
   ! initialize vector module
   call FNVInitS(4, NEQ, ier)
 
+  ! initialize system and mass matrix modules
+  nnz = 15*NEQ     ! integer number of nonzeros           
+  ordering = 0     ! AMD
+  sparsetype = 1   ! CSR
+  call FSunSparseMatInit(4, NEQ, NEQ, nnz, sparsetype, ier)
+  call FSunSparseMassMatInit(NEQ, NEQ, nnz, sparsetype, ier)
+
+  ! initialize KLU system and mass solvers
+  call FSunKLUInit(4, ier)
+  call FSunMassKLUInit(ier)
+  
   ! initialize ARKode solver
   ipar = 0
   rpar = 0.0
@@ -248,18 +263,17 @@ program driver
                   iout, rout, ipar, rpar, ier)
 
   ! set optional inputs
-  call FARKSetIin('IMPLICIT', 1, ier)
-  call FARKSetIin('MAX_NSTEPS', 1000, ier)
+  Iinput = 1
+  call FARKSetIin('IMPLICIT', Iinput, ier)
+  Iinput = 1000
+  call FARKSetIin('MAX_NSTEPS', Iinput, ier)
   call FARKSetResTolerance(1, atol, ier)
 
-  ! specify use of KLU linear solvers
-  ineq = NEQ       ! convert to 'normal' integer type
-  nnz = 15*NEQ     ! integer number of nonzeros           
-  ordering = 0     ! AMD
-  sparsetype = 1   ! CSR
-  call FARKKLU(ineq, nnz, sparsetype, ordering, ier)
+  ! attach matrix and linear solver objects to ARKDls interfaces
+  time_dep = 0
+  call FARKDlsInit(ier)
   call FARKSparseSetJac(ier)
-  call FARKMassKLU(ineq, nnz, sparsetype, ordering, ier)
+  call FARKDlsMassInit(time_dep, ier)
   call FARKSparseSetMass(ier)
 
   ! Open output stream for results
@@ -355,11 +369,11 @@ subroutine FARKIFun(t, y, ydot, ipar, rpar, ier)
   implicit none
 
   ! Arguments
-  real*8,  intent(in)   :: t, rpar(1)
+  real(kind=REALTYPE), intent(in)  :: t, rpar(1)
   integer*8, intent(in) :: ipar(1)
   integer, intent(out)  :: ier
-  real*8,  intent(in)   :: y(3,N)
-  real*8,  intent(out)  :: ydot(3,N)
+  real(kind=REALTYPE), intent(in)  :: y(3,N)
+  real(kind=REALTYPE), intent(out) :: ydot(3,N)
 
   ! Local data
   integer :: ix
@@ -534,11 +548,11 @@ subroutine farkefun(t, y, ydot, ipar, rpar, ier)
   implicit none
 
   ! Arguments
-  real*8,  intent(in)   :: t, rpar(1)
+  real(kind=REALTYPE), intent(in)  :: t, rpar(1)
   integer*8, intent(in) :: ipar(1)
+  real(kind=REALTYPE), intent(in)  :: y(3,N)
+  real(kind=REALTYPE), intent(out) :: ydot(3,N)
   integer, intent(out)  :: ier
-  real*8,  intent(in)   :: y(3,N)
-  real*8,  intent(out)  :: ydot(3,N)
 
   ! return with success (since fully implicit)
   ydot = 0.d0
@@ -562,15 +576,16 @@ subroutine farkspjac(t, y, fy, neq, nnz, Jdata, Jcolvals, &
   implicit none
 
   ! Arguments
-  real*8,    intent(in)  :: t, h, rpar(1)
-  real*8,    intent(in), dimension(3,N) :: y, fy, wk1, wk2, wk3
-  integer,   intent(in)  :: neq, nnz
-  integer*8, intent(in)  :: ipar(1)
-  real*8,    intent(out) :: Jdata(nnz)
-  integer,   intent(out) :: Jcolvals(nnz), Jrowptrs(neq+1), ier
+  real(kind=REALTYPE), intent(in)  :: t, h, rpar(1)
+  real(kind=REALTYPE), intent(in), dimension(3,N) :: y, fy, wk1, wk2, wk3
+  real(kind=REALTYPE), intent(out) :: Jdata(nnz)
+  integer*8, intent(in)  :: ipar(1), neq, nnz
+  integer(kind=SUNINDEXTYPE), intent(out) :: Jcolvals(nnz)
+  integer(kind=SUNINDEXTYPE), intent(out) :: Jrowptrs(neq+1)
+  integer,   intent(out) :: ier
 
   ! Local data
-  integer :: ix, nz
+  integer :: ix, nz, Nint
   real*8  :: ul, uc, ur, vl, vc, vr, wl, wc, wr, xl, xc, xr
   real*8  :: u1, u2, u3, v1, v2, v3, w1, w2, w3
   real*8  :: f1, f2, f3, df1, df2, df3, dQdf1, dQdf2, dQdf3
@@ -583,6 +598,9 @@ subroutine farkspjac(t, y, fy, neq, nnz, Jdata, Jcolvals, &
      return
   endif
 
+  ! set integer*4 version of N for call to idx()
+  Nint = N
+  
   ! clear out Jacobian matrix data
   Jdata = 0.d0
   nz = 0
@@ -968,12 +986,12 @@ subroutine farkspjac(t, y, fy, neq, nnz, Jdata, Jcolvals, &
   enddo
 
   ! Dirichlet boundary at right
-  Jrowptrs(idx(N,1)+1) = nz
-  Jrowptrs(idx(N,2)+1) = nz
-  Jrowptrs(idx(N,3)+1) = nz
+  Jrowptrs(idx(Nint,1)+1) = nz
+  Jrowptrs(idx(Nint,2)+1) = nz
+  Jrowptrs(idx(Nint,3)+1) = nz
  
   ! signal end of data in CSR matrix
-  Jrowptrs(idx(N,3)+2) = nz
+  Jrowptrs(idx(Nint,3)+2) = nz
 
   ier = 0
   return
@@ -996,15 +1014,16 @@ subroutine farkspmass(t, neq, nnz, Mdata, Mcolvals, Mrowptrs, &
   implicit none
 
   ! Arguments
-  real*8,    intent(in)  :: t, rpar(1)
-  real*8,    intent(in), dimension(3,N) :: wk1, wk2, wk3
-  integer,   intent(in)  :: neq, nnz
-  integer*8, intent(in)  :: ipar(1)
-  real*8,    intent(out) :: Mdata(nnz)
-  integer,   intent(out) :: Mcolvals(nnz), Mrowptrs(neq+1), ier
+  real(kind=REALTYPE), intent(in)  :: t, rpar(1)
+  real(kind=REALTYPE), intent(in), dimension(3,N) :: wk1, wk2, wk3
+  real(kind=REALTYPE), intent(out) :: Mdata(nnz)
+  integer*8, intent(in) :: ipar(1), neq, nnz
+  integer(kind=SUNINDEXTYPE), intent(out) :: Mcolvals(nnz)
+  integer(kind=SUNINDEXTYPE), intent(out) :: Mrowptrs(neq+1)
+  integer,  intent(out) :: ier
 
   ! Local data
-  integer :: ix, nz
+  integer :: ix, nz, Nint
   real*8  :: xl, xc, xr, Ml, Mc, Mr, ChiL1, ChiL2, ChiL3, ChiR1, ChiR2, ChiR3
   logical :: left, right
 
@@ -1014,11 +1033,14 @@ subroutine farkspmass(t, neq, nnz, Mdata, Mcolvals, Mrowptrs, &
      return
   endif
 
+  ! set integer*4 version of N for call to idx()
+  Nint = N
+  
   ! clear out Jacobian matrix data
   Mdata = 0.d0
+  nz = 0
 
   ! iterate through nodes, filling in matrix by rows
-  nz = 0
   do ix=1,N
 
      ! set booleans to determine whether intervals exist on the left/right */
@@ -1125,7 +1147,7 @@ subroutine farkspmass(t, neq, nnz, Mdata, Mcolvals, Mrowptrs, &
   enddo
 
   ! signal end of data in CSR matrix
-  Mrowptrs(idx(N,3)+2) = nz
+  Mrowptrs(idx(Nint,3)+2) = nz
 
   ier = 0
   return
