@@ -1,7 +1,4 @@
 /*
- * -----------------------------------------------------------------
- * $Revision$
- * $Date$
  * -----------------------------------------------------------------
  * Programmer(s): Jean M. Sexton @ SMU
  *                Slaven Peles @ LLNL
@@ -75,8 +72,8 @@ typedef struct {
 
 /* Private Helper Functions */
 
-static void SetIC(HYPRE_IJVector Uij, realtype dx, long int my_length,
-                  long int my_base);
+static void SetIC(HYPRE_IJVector Uij, realtype dx, sunindextype my_length,
+                  sunindextype my_base);
 
 static void PrintIntro(int npes);
 
@@ -144,7 +141,7 @@ int main(int argc, char *argv[])
   data->hdcoef = RCONST(1.0)/(dx*dx);
   data->hacoef = RCONST(0.5)/(RCONST(2.0)*dx);
 
-  /* Initialize solutin vector. */
+  /* Initialize solution vector. */
   SetIC(Uij, dx, local_N, my_base);
   HYPRE_IJVectorAssemble(Uij);
   HYPRE_IJVectorGetObject(Uij, (void**) &Upar);
@@ -208,8 +205,8 @@ int main(int argc, char *argv[])
 
 /* Set initial conditions in u vector */
 
-static void SetIC(HYPRE_IJVector Uij, realtype dx, long int my_length,
-                  long int my_base)
+static void SetIC(HYPRE_IJVector Uij, realtype dx, sunindextype my_length,
+                  sunindextype my_base)
 {
   int i;
   HYPRE_Int *iglobal;
