@@ -1,8 +1,4 @@
-/*
- * -----------------------------------------------------------------
- * $Revision: 4868 $
- * $Date: 2016-08-19 10:16:31 -0700 (Fri, 19 Aug 2016) $
- * -----------------------------------------------------------------
+/* -----------------------------------------------------------------
  * Programmer(s): Scott D. Cohen, Alan C. Hindmarsh and
  *                Radu Serban @ LLNL
  * -----------------------------------------------------------------
@@ -30,28 +26,27 @@
  * called for each of the remaining seven runs.
  *
  * Notes: This program demonstrates the usage of the sequential
- * macros NV_Ith_S, DENSE_ELEM, BAND_COL, and
- * BAND_COL_ELEM. The NV_Ith_S macro is used to reference the
+ * macros NV_Ith_S, SM_ELEMENT_D, SM_COLUMN_B, and
+ * SM_COLUMN_ELEMENT_B. The NV_Ith_S macro is used to reference the
  * components of an N_Vector. It works for any size N=NEQ, but
  * due to efficiency concerns it should only by used when the
  * problem size is small. The Problem 1 right hand side and
  * Jacobian functions f1 and Jac1 both use NV_Ith_S. The 
- * N_VGetArrayPointer_Serial function gives the user access to the 
+ * N_VGetArrayPointer function gives the user access to the 
  * memory used for the component storage of an N_Vector. In the 
  * sequential case, the user may assume that this is one contiguous 
- * array of reals. The N_VGetArrayPointer_Serial function
+ * array of reals. The N_VGetArrayPointer function
  * gives a more efficient means (than the NV_Ith_S macro) to
  * access the components of an N_Vector and should be used when the
  * problem size is large. The Problem 2 right hand side function f2
- * uses the N_VGetArrayPointer_Serial function. The DENSE_ELEM macro 
- * used in Jac1 gives access to an element of a dense matrix of type 
- * DlsMat. It should be used only when the problem size is small (the 
- * size of a DlsMat is NEQ x NEQ) due to efficiency concerns. For
- * larger problem sizes, the macro DENSE_COL can be used in order
- * to work directly with a column of a DlsMat. The BAND_COL and
- * BAND_COL_ELEM allow efficient columnwise access to the elements
- * of a band matrix of type DlsMat. These macros are used in the
- * Jac2 function.
+ * uses the N_VGetArrayPointer function. The SM_ELEMENT_D macro 
+ * used in Jac1 gives access to an element of a dense SUNMatrix. It 
+ * should be used only when the problem size is small (the 
+ * size of a Dense SUNMatrix is NEQ x NEQ) due to efficiency concerns. For
+ * larger problem sizes, the macro SM_COLUMN_D can be used in order
+ * to work directly with a column of a Dense SUNMatrix. The SM_COLUMN_B and
+ * SM_COLUMN_ELEMENT_B allow efficient columnwise access to the elements
+ * of a Banded SUNMatix. These macros are used in the Jac2 function.
  * -----------------------------------------------------------------
  */
 
@@ -59,13 +54,16 @@
 #include <stdlib.h>
 #include <math.h>
 
-#include <cvodes/cvodes.h>             /* main integrator header file */
-#include <cvodes/cvodes_dense.h>       /* use CVDENSE linear solver */
-#include <cvodes/cvodes_band.h>        /* use CVBAND linear solver */
-#include <cvodes/cvodes_diag.h>        /* use CVDIAG linear solver */
-#include <nvector/nvector_serial.h>  /* serial N_Vector types, fct. and macros */
-#include <sundials/sundials_types.h> /* definition of realtype */
-#include <sundials/sundials_math.h>  /* contains the macros ABS, SUNSQR, and EXP*/
+#include <cvodes/cvodes.h>                /* prototypes for CVODE fcts., consts.      */
+#include <nvector/nvector_serial.h>       /* access to serial N_Vector                */
+#include <sunmatrix/sunmatrix_dense.h>    /* access to dense SUNMatrix                */
+#include <sunlinsol/sunlinsol_dense.h>    /* access to dense SUNLinearSolver          */
+#include <sunmatrix/sunmatrix_band.h>     /* access to band SUNMatrix                 */
+#include <sunlinsol/sunlinsol_band.h>     /* access to band SUNLinearSolver           */
+#include <cvodes/cvodes_direct.h>         /* access to CVDls interface                */
+#include <cvodes/cvodes_diag.h>           /* access to CVDIAG linear solver interface */
+#include <sundials/sundials_types.h>      /* definition of realtype                   */
+#include <sundials/sundials_math.h>       /* contains the macros ABS, SUNSQR, and EXP */
 
 /* Shared Problem Constants */
 
@@ -117,8 +115,9 @@ static void PrintIntro2(void);
 static void PrintHeader2(void);
 static void PrintOutput2(realtype t, realtype erm, int qu, realtype hu);
 static realtype MaxError(N_Vector y, realtype t);
-static int PrepareNextRun(void *cvode_mem, int lmm, int miter, long int mu,
-                          long int ml);
+static int PrepareNextRun(void *cvode_mem, int lmm, int miter, N_Vector y, 
+                          SUNMatrix A, sunindextype mu, sunindextype ml,
+                          SUNLinearSolver LS);
 static void PrintErrOutput(realtype tol_factor);
 static void PrintFinalStats(void *cvode_mem, int miter, realtype ero);
 static void PrintErrInfo(int nerr);
@@ -126,14 +125,11 @@ static void PrintErrInfo(int nerr);
 /* Functions Called by the Solver */
 
 static int f1(realtype t, N_Vector y, N_Vector ydot, void *user_data);
-static int Jac1(long int N, realtype tn,
-                N_Vector y, N_Vector fy, 
-                DlsMat J, void *user_data,
-                N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
+static int Jac1(realtype tn, N_Vector y, N_Vector fy, SUNMatrix J,
+                void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
+
 static int f2(realtype t, N_Vector y, N_Vector ydot, void *user_data);
-static int Jac2(long int N, long int mu, long int ml, 
-                realtype tn, N_Vector y, N_Vector fy, 
-                DlsMat J, void *user_data,
+static int Jac2(realtype tn, N_Vector y, N_Vector fy, SUNMatrix J, void *user_data,
                 N_Vector tmp1, N_Vector tmp2, N_Vector tmp3);
 
 /* Private function to check function return values */
@@ -158,12 +154,16 @@ static int Problem1(void)
   realtype reltol=RTOL, abstol=ATOL, t, tout, ero, er;
   int miter, flag, temp_flag, iout, nerr=0;
   N_Vector y;
+  SUNMatrix A;
+  SUNLinearSolver LS;
   void *cvode_mem;
   booleantype firstrun;
   int qu;
   realtype hu;
 
   y = NULL;
+  A = NULL;
+  LS = NULL;
   cvode_mem = NULL;
 
   y = N_VNew_Serial(P1_NEQ);
@@ -191,7 +191,7 @@ static int Problem1(void)
       if(check_flag(&flag, "CVodeReInit", 1)) return(1);
     }
       
-    flag = PrepareNextRun(cvode_mem, CV_ADAMS, miter, 0, 0);
+    flag = PrepareNextRun(cvode_mem, CV_ADAMS, miter, y, A, 0, 0, LS);
     if(check_flag(&flag, "PrepareNextRun", 1)) return(1);
 
     PrintHeader1();
@@ -244,7 +244,7 @@ static int Problem1(void)
       if(check_flag(&flag, "CVodeReInit", 1)) return(1);
     }
       
-    flag = PrepareNextRun(cvode_mem, CV_BDF, miter, 0, 0);     
+    flag = PrepareNextRun(cvode_mem, CV_BDF, miter, y, A, 0, 0, LS);     
     if(check_flag(&flag, "PrepareNextRun", 1)) return(1);
 
     PrintHeader1();
@@ -275,7 +275,7 @@ static int Problem1(void)
   }
 
   CVodeFree(&cvode_mem);
-  N_VDestroy_Serial(y);
+  N_VDestroy(y);
 
   return(nerr);
 }
@@ -331,19 +331,17 @@ static int f1(realtype t, N_Vector y, N_Vector ydot, void *user_data)
   return(0);
 } 
 
-static int Jac1(long int N, realtype tn,
-                N_Vector y, N_Vector fy, 
-                DlsMat J, void *user_data,
-                N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
+static int Jac1(realtype tn, N_Vector y, N_Vector fy, SUNMatrix J, 
+                void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
   realtype y0, y1;
 
   y0 = NV_Ith_S(y,0);
   y1 = NV_Ith_S(y,1);
 
-  DENSE_ELEM(J,0,1) = ONE;
-  DENSE_ELEM(J,1,0) = -TWO * P1_ETA * y0 * y1 - ONE;
-  DENSE_ELEM(J,1,1) = P1_ETA * (ONE - SUNSQR(y0));
+  SM_ELEMENT_D(J,0,1) = ONE;
+  SM_ELEMENT_D(J,1,0) = -TWO * P1_ETA * y0 * y1 - ONE;
+  SM_ELEMENT_D(J,1,1) = P1_ETA * (ONE - SUNSQR(y0));
 
   return(0);
 }
@@ -353,16 +351,20 @@ static int Problem2(void)
   realtype reltol=RTOL, abstol=ATOL, t, tout, er, erm, ero;
   int miter, flag, temp_flag, nerr=0;
   N_Vector y;
+  SUNMatrix A;
+  SUNLinearSolver LS;
   void *cvode_mem;
   booleantype firstrun;
   int qu, iout;
   realtype hu;
 
   y = NULL;
+  A = NULL;
+  LS = NULL;
   cvode_mem = NULL;
 
   y = N_VNew_Serial(P2_NEQ);
-  if(check_flag((void *)y, "N_VNew", 0)) return(1);
+  if(check_flag((void *)y, "N_VNew_Serial", 0)) return(1);
 
   PrintIntro2();
 
@@ -388,7 +390,7 @@ static int Problem2(void)
       if(check_flag(&flag, "CVodeReInit", 1)) return(1);
     }
       
-    flag = PrepareNextRun(cvode_mem, CV_ADAMS, miter, P2_MU, P2_ML);
+    flag = PrepareNextRun(cvode_mem, CV_ADAMS, miter, y, A, P2_MU, P2_ML, LS);
     if(check_flag(&flag, "PrepareNextRun", 1)) return(1);
 
     PrintHeader2();
@@ -441,7 +443,7 @@ static int Problem2(void)
       if(check_flag(&flag, "CVodeReInit", 1)) return(1);
     }
 
-    flag = PrepareNextRun(cvode_mem, CV_BDF, miter, P2_MU, P2_ML);
+    flag = PrepareNextRun(cvode_mem, CV_BDF, miter, y, A, P2_MU, P2_ML, LS);
     if(check_flag(&flag, "PrepareNextRun", 1)) return(1);
 
     PrintHeader2();
@@ -471,7 +473,7 @@ static int Problem2(void)
   }
 
   CVodeFree(&cvode_mem);
-  N_VDestroy_Serial(y);
+  N_VDestroy(y);
 
   return(nerr);
 }
@@ -515,11 +517,11 @@ static void PrintOutput2(realtype t, realtype erm, int qu, realtype hu)
 
 static int f2(realtype t, N_Vector y, N_Vector ydot, void *user_data)
 {
-  long int i, j, k;
+  sunindextype i, j, k;
   realtype d, *ydata, *dydata;
   
-  ydata = N_VGetArrayPointer_Serial(y);
-  dydata = N_VGetArrayPointer_Serial(ydot);
+  ydata = N_VGetArrayPointer(y);
+  dydata = N_VGetArrayPointer(ydot);
 
   /*
      Excluding boundaries, 
@@ -541,10 +543,8 @@ static int f2(realtype t, N_Vector y, N_Vector ydot, void *user_data)
   return(0);
 }
 
-static int Jac2(long int N, long int mu, long int ml, 
-                realtype tn, N_Vector y, N_Vector fy, 
-                DlsMat J, void *user_data,
-                N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
+static int Jac2(realtype tn, N_Vector y, N_Vector fy, SUNMatrix J, 
+                void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
   int i, j, k;
   realtype *kthCol;
@@ -568,10 +568,10 @@ static int Jac2(long int N, long int mu, long int ml,
   for (j=0; j < P2_MESHY; j++) {
     for (i=0; i < P2_MESHX; i++) {
       k = i + j * P2_MESHX;
-      kthCol = BAND_COL(J,k);
-      BAND_COL_ELEM(kthCol,k,k) = -TWO;
-      if (i != P2_MESHX-1) BAND_COL_ELEM(kthCol,k+1,k) = P2_ALPH1;
-      if (j != P2_MESHY-1) BAND_COL_ELEM(kthCol,k+P2_MESHX,k) = P2_ALPH2;
+      kthCol = SM_COLUMN_B(J,k);
+      SM_COLUMN_ELEMENT_B(kthCol,k,k) = -TWO;
+      if (i != P2_MESHX-1) SM_COLUMN_ELEMENT_B(kthCol,k+1,k) = P2_ALPH1;
+      if (j != P2_MESHY-1) SM_COLUMN_ELEMENT_B(kthCol,k+P2_MESHX,k) = P2_ALPH2;
     }
   }
 
@@ -580,12 +580,12 @@ static int Jac2(long int N, long int mu, long int ml,
 
 static realtype MaxError(N_Vector y, realtype t)
 {
-  long int i, j, k;
+  sunindextype i, j, k;
   realtype *ydata, er, ex=ZERO, yt, maxError=ZERO, ifact_inv, jfact_inv=ONE;
   
   if (t == ZERO) return(ZERO);
 
-  ydata = N_VGetArrayPointer_Serial(y);
+  ydata = N_VGetArrayPointer(y);
   if (t <= THIRTY) ex = SUNRexp(-TWO*t);
   
   for (j = 0; j < P2_MESHY; j++) {
@@ -602,8 +602,9 @@ static realtype MaxError(N_Vector y, realtype t)
   return(maxError);
 }
 
-static int PrepareNextRun(void *cvode_mem, int lmm, int miter, 
-                          long int mu, long int ml)
+static int PrepareNextRun(void *cvode_mem, int lmm, int miter, N_Vector y, 
+                          SUNMatrix A, sunindextype mu, sunindextype ml,
+                          SUNLinearSolver LS)
 {
   int flag = CV_SUCCESS;
   
@@ -622,38 +623,96 @@ static int PrepareNextRun(void *cvode_mem, int lmm, int miter,
   } else {
     printf("NEWTON\n");
     printf("Linear Solver           : ");
+
     switch(miter) {
+
     case DENSE_USER : 
       printf("Dense, User-Supplied Jacobian\n");
-      flag = CVDense(cvode_mem, P1_NEQ);
-      check_flag(&flag, "CVDense", 1);
-      if(flag != CV_SUCCESS) break;
-      flag = CVDlsSetDenseJacFn(cvode_mem, Jac1);
-      check_flag(&flag, "CVDlsSetDenseJacFn", 1);
+
+      /* Create dense SUNMatrix for use in linear solves */
+      A = SUNDenseMatrix(P1_NEQ, P1_NEQ);
+      if(check_flag((void *)A, "SUNDenseMatrix", 0)) return(1);
+
+      /* Create dense SUNLinearSolver object for use by CVode */
+      LS = SUNDenseLinearSolver(y, A);
+      if(check_flag((void *)LS, "SUNDenseLinearSolver", 0)) return(1);
+
+      /* Call CVDlsSetLinearSolver to attach the matrix and linear solver to CVode */
+      flag = CVDlsSetLinearSolver(cvode_mem, LS, A);
+      if(check_flag(&flag, "CVDlsSetLinearSolver", 1)) return(1);
+
+      /* Set the user-supplied Jacobian routine Jac */
+      flag = CVDlsSetJacFn(cvode_mem, Jac1);
+      if(check_flag(&flag, "CVDlsSetJacFn", 1)) return(1);
       break;
-    case DENSE_DQ   : 
+
+    case DENSE_DQ : 
       printf("Dense, Difference Quotient Jacobian\n");
-      flag = CVDlsSetDenseJacFn(cvode_mem, NULL);
-      check_flag(&flag, "CVDlsSetDenseJacFn", 1);
+
+      /* Create dense SUNMatrix for use in linear solves */
+      A = SUNDenseMatrix(P1_NEQ, P1_NEQ);
+      if(check_flag((void *)A, "SUNDenseMatrix", 0)) return(1);
+
+      /* Create dense SUNLinearSolver object for use by CVode */
+      LS = SUNDenseLinearSolver(y, A);
+      if(check_flag((void *)LS, "SUNDenseLinearSolver", 0)) return(1);
+
+      /* Call CVDlsSetLinearSolver to attach the matrix and linear solver to CVode */
+      flag = CVDlsSetLinearSolver(cvode_mem, LS, A);
+      if(check_flag(&flag, "CVDlsSetLinearSolver", 1)) return(1);
+
+      /* Use a difference quotient Jacobian */
+      flag = CVDlsSetJacFn(cvode_mem, NULL);
+      if(check_flag(&flag, "CVDlsSetJacFn", 1)) return(1);
       break;
-    case DIAG       : 
+
+    case DIAG : 
       printf("Diagonal Jacobian\n");
+     
+      /* Call CVDiag to initialize and attach the CVode-specific diagonal linear solver */
       flag = CVDiag(cvode_mem);
-      check_flag(&flag, "CVDiag", 1);
+      if(check_flag(&flag, "CVDiag", 1)) return(1);
       break;
-    case BAND_USER  : 
+
+    case BAND_USER : 
       printf("Band, User-Supplied Jacobian\n");
-      flag = CVBand(cvode_mem, P2_NEQ, mu, ml);
-      check_flag(&flag, "CVBand", 1);
-      if(flag != CV_SUCCESS) break;
-      flag = CVDlsSetBandJacFn(cvode_mem, Jac2);
-      check_flag(&flag, "CVDlsSetBandJacFn", 1);
+
+      /* Create band SUNMatrix for use in linear solves */
+      A = SUNBandMatrix(P2_NEQ, mu, ml, mu+ml);
+      if(check_flag((void *)A, "SUNBandMatrix", 0)) return(1);
+
+      /* Create banded SUNLinearSolver object for use by CVode */
+      LS = SUNBandLinearSolver(y, A);
+      if(check_flag((void *)LS, "SUNBandLinearSolver", 0)) return(1);
+
+      /* Call CVDlsSetLinearSolver to attach the matrix and linear solver to CVode */
+      flag = CVDlsSetLinearSolver(cvode_mem, LS, A);
+      if(check_flag(&flag, "CVDlsSetLinearSolver", 1)) return(1);
+
+      /* Set the user-supplied Jacobian routine Jac */
+      flag = CVDlsSetJacFn(cvode_mem, Jac2);
+      if(check_flag(&flag, "CVDlsSetJacFn", 1)) return(1);
       break;
+
     case BAND_DQ  :   
       printf("Band, Difference Quotient Jacobian\n");
-      flag = CVDlsSetBandJacFn(cvode_mem, NULL);
-      check_flag(&flag, "CVDlsSetBandJacFn", 1);
-      break;    
+
+      /* Create band SUNMatrix for use in linear solves */
+      A = SUNBandMatrix(P2_NEQ, mu, ml, mu+ml);
+      if(check_flag((void *)A, "SUNBandMatrix", 0)) return(1);
+
+      /* Create banded SUNLinearSolver object for use by CVode */
+      LS = SUNBandLinearSolver(y, A);
+      if(check_flag((void *)LS, "SUNBandLinearSolver", 0)) return(1);
+
+      /* Call CVDlsSetLinearSolver to attach the matrix and linear solver to CVode */
+      flag = CVDlsSetLinearSolver(cvode_mem, LS, A);
+      if(check_flag(&flag, "CVDlsSetLinearSolver", 1)) return(1);
+
+      /* Use a difference quotient Jacobian */
+      flag = CVDlsSetJacFn(cvode_mem, NULL);
+      if(check_flag(&flag, "CVDlsSetJacFn", 1)) return(1);
+      break;
     }
   }
 
@@ -675,8 +734,8 @@ static void PrintErrOutput(realtype tol_factor)
 
 static void PrintFinalStats(void *cvode_mem, int miter, realtype ero)
 {
-  long int lenrw, leniw, nst, nfe, nsetups, nni, ncfn, netf;
-  long int lenrwLS, leniwLS, nje, nfeLS;
+  long int lenrw, leniw, lenrwLS, leniwLS;
+  long int nst, nfe, nsetups, nni, ncfn, netf, nje, nfeLS;
   int flag;
 
   flag = CVodeGetWorkSpace(cvode_mem, &lenrw, &leniw);
@@ -695,8 +754,8 @@ static void PrintFinalStats(void *cvode_mem, int miter, realtype ero)
   check_flag(&flag, "CVodeGetNumNonlinSolvConvFails", 1);
 
   printf("\n Final statistics for this run:\n\n");
-  printf(" CVode real workspace length              = %4ld \n", lenrw);
-  printf(" CVode integer workspace length           = %4ld \n", leniw);
+  printf(" CVode real workspace length              = %4ld \n",  lenrw);
+  printf(" CVode integer workspace length           = %4ld \n",  leniw);
   printf(" Number of steps                          = %4ld \n",  nst);
   printf(" Number of f-s                            = %4ld \n",  nfe);
   printf(" Number of setups                         = %4ld \n",  nsetups);
@@ -705,36 +764,23 @@ static void PrintFinalStats(void *cvode_mem, int miter, realtype ero)
   printf(" Number of error test failures            = %4ld \n\n",netf);
   
   if (miter != FUNC) {
-    switch(miter) {
-    case DENSE_USER :
-    case DENSE_DQ   :
+    if (miter != DIAG) {
       flag = CVDlsGetNumJacEvals(cvode_mem, &nje);
       check_flag(&flag, "CVDlsGetNumJacEvals", 1);
       flag = CVDlsGetNumRhsEvals(cvode_mem, &nfeLS);
       check_flag(&flag, "CVDlsGetNumRhsEvals", 1);
       flag = CVDlsGetWorkSpace(cvode_mem, &lenrwLS, &leniwLS);
       check_flag(&flag, "CVDlsGetWorkSpace", 1);
-      break;
-    case BAND_USER  :
-    case BAND_DQ    :
-      flag = CVDlsGetNumJacEvals(cvode_mem, &nje);
-      check_flag(&flag, "CVDlsGetNumJacEvals", 1);
-      flag = CVDlsGetNumRhsEvals(cvode_mem, &nfeLS);
-      check_flag(&flag, "CVDlsGetNumRhsEvals", 1);
-      flag = CVDlsGetWorkSpace(cvode_mem, &lenrwLS, &leniwLS);
-      check_flag(&flag, "CVDlsGetWorkSpace", 1);
-      break;  
-    case DIAG       :
+    } else {
       nje = nsetups;
       flag = CVDiagGetNumRhsEvals(cvode_mem, &nfeLS);
       check_flag(&flag, "CVDiagGetNumRhsEvals", 1);
       flag = CVDiagGetWorkSpace(cvode_mem, &lenrwLS, &leniwLS);
       check_flag(&flag, "CVDiagGetWorkSpace", 1);
-      break;
     }
     printf(" Linear solver real workspace length      = %4ld \n", lenrwLS);
     printf(" Linear solver integer workspace length   = %4ld \n", leniwLS);
-    printf(" Number of Jacobian evaluations           = %4ld  \n", nje);
+    printf(" Number of Jacobian evaluations           = %4ld \n", nje);
     printf(" Number of f evals. in linear solver      = %4ld \n\n", nfeLS);
   }
   
