@@ -254,9 +254,9 @@ static int g(realtype t, N_Vector y, realtype *gout, void *user_data)
 static int Jac(realtype t, N_Vector y, N_Vector fy, SUNMatrix J, 
                void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
-  realtype y1, y2, y3;
+  realtype y2, y3;
 
-  y1 = Ith(y,1); y2 = Ith(y,2); y3 = Ith(y,3);
+  y2 = Ith(y,2); y3 = Ith(y,3);
 
   IJth(J,1,1) = RCONST(-0.04);
   IJth(J,1,2) = RCONST(1.0e4)*y3;
@@ -283,10 +283,8 @@ static void PrintOutput(realtype t, realtype y1, realtype y2, realtype y3)
 {
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("At t = %0.4Le      y =%14.6Le  %14.6Le  %14.6Le\n", t, y1, y2, y3);
-#elif defined(SUNDIALS_DOUBLE_PRECISION)
-  printf("At t = %0.4le      y =%14.6le  %14.6le  %14.6le\n", t, y1, y2, y3);
 #else
-  printf("At t = %0.4e      y =%14.6e  %14.6e  %14.6e\n", t, y1, y2, y3);
+  printf("At t = %0.4e       y =%14.6e   %14.6e   %14.6e\n", t, y1, y2, y3);
 #endif
 
   return;
