@@ -523,12 +523,12 @@ int main(int argc, char *argv[])
 
 static int res(realtype tres, N_Vector yy, N_Vector yp, N_Vector rr, void *user_data)
 {
-  realtype y1, y2, y3, yp1, yp2, yp3, *rval;
+  realtype y1, y2, y3, yp1, yp2, *rval;
   UserData data;
   realtype p1, p2, p3;
 
   y1  = Ith(yy,1); y2  = Ith(yy,2); y3  = Ith(yy,3); 
-  yp1 = Ith(yp,1); yp2 = Ith(yp,2); yp3 = Ith(yp,3);
+  yp1 = Ith(yp,1); yp2 = Ith(yp,2);
   rval = N_VGetArrayPointer(rr);
 
   data = (UserData) user_data;
@@ -551,9 +551,8 @@ static int resS(int Ns, realtype t,
   UserData data;
   realtype p1, p2, p3;
   realtype y1, y2, y3;
-  realtype yp1, yp2, yp3;
   realtype s1, s2, s3;
-  realtype sd1, sd2, sd3;
+  realtype sd1, sd2;
   realtype rs1, rs2, rs3;
   int is;
 
@@ -566,10 +565,6 @@ static int resS(int Ns, realtype t,
   y2 = Ith(yy,2);
   y3 = Ith(yy,3);
 
-  yp1 = Ith(yp,1);
-  yp2 = Ith(yp,2);
-  yp3 = Ith(yp,3);
-
   for (is=0; is<NP; is++) {
 
     s1 = Ith(yyS[is],1);
@@ -578,7 +573,6 @@ static int resS(int Ns, realtype t,
 
     sd1 = Ith(ypS[is],1);
     sd2 = Ith(ypS[is],2);
-    sd3 = Ith(ypS[is],3);
 
     rs1 = sd1 + p1*s1 - p2*y3*s2 - p2*y2*s3;
     rs2 = sd2 - p1*s1 + p2*y3*s2 + p2*y2*s3 + TWO*p3*y2*s2;
@@ -702,25 +696,19 @@ static int rhsQBS1(realtype tt,
                  N_Vector yyB, N_Vector ypB,
                  N_Vector rhsBQS, void *user_dataB)
 {
-  UserData data;
   realtype y1, y2, y3;
-  realtype p1, p2, p3;
-  realtype l1, l2, l3, m1, m2, m3;
+  realtype l1, l2, m1, m2;
   realtype s1, s2, s3;
   realtype l21;
-  
-  data = (UserData) user_dataB;
-
-  /* The p vector */
-  p1 = data->p[0]; p2 = data->p[1]; p3 = data->p[2];
 
   /* The y vector */
   y1 = Ith(yy,1); y2 = Ith(yy,2); y3 = Ith(yy,3);
   
   /* The lambda vector. */
-  l1 = Ith(yyB,1); l2 = Ith(yyB,2); l3 = Ith(yyB,3);
+  l1 = Ith(yyB,1); l2 = Ith(yyB,2);
+
   /* The mu vector. */
-  m1 = Ith(yyB,4); m2 = Ith(yyB,5); m3 = Ith(yyB,6);
+  m1 = Ith(yyB,4); m2 = Ith(yyB,5);
 
   /* The sensitivity with respect to p1 */
   s1 = Ith(yyS[0],1); s2 = Ith(yyS[0],2); s3 = Ith(yyS[0],3);
@@ -767,6 +755,7 @@ static int resBS2(realtype tt,
 
   /* The lambda dot vector. */
   lp1 = Ith(ypB,1); lp2 = Ith(ypB,2);
+
   /* The mu dot vector. */
   mp1 = Ith(ypB,4); mp2 = Ith(ypB,5);
 
@@ -793,25 +782,19 @@ static int rhsQBS2(realtype tt,
                  N_Vector yyB, N_Vector ypB,
                  N_Vector rhsBQS, void *user_dataB)
 {
-  UserData data;
   realtype y1, y2, y3;
-  realtype p1, p2, p3;
-  realtype l1, l2, l3, m1, m2, m3;
+  realtype l1, l2, m1, m2;
   realtype s1, s2, s3;
   realtype l21;
-  
-  data = (UserData) user_dataB;
-
-  /* The p vector */
-  p1 = data->p[0]; p2 = data->p[1]; p3 = data->p[2];
 
   /* The y vector */
   y1 = Ith(yy,1); y2 = Ith(yy,2); y3 = Ith(yy,3);
   
   /* The lambda vector. */
-  l1 = Ith(yyB,1); l2 = Ith(yyB,2); l3 = Ith(yyB,3);
+  l1 = Ith(yyB,1); l2 = Ith(yyB,2);
+
   /* The mu vector. */
-  m1 = Ith(yyB,4); m2 = Ith(yyB,5); m3 = Ith(yyB,6);
+  m1 = Ith(yyB,4); m2 = Ith(yyB,5);
 
   /* The sensitivity with respect to p2 */
   s1 = Ith(yyS[1],1); s2 = Ith(yyS[1],2); s3 = Ith(yyS[1],3);
