@@ -225,7 +225,6 @@ static int func(N_Vector u, N_Vector f, void *user_data)
   realtype hdc, vdc;
   realtype uij, udn, uup, ult, urt;
   realtype *udata, *fdata;
-  realtype x,y;
 
   int i, j;
 
@@ -238,13 +237,8 @@ static int func(N_Vector u, N_Vector f, void *user_data)
   fdata = N_VGetArrayPointer_Serial(f);
 
   for (j=1; j <= NY; j++) {
-
-    y = j*dy;
-
     for (i=1; i <= NX; i++) {
 
-      x = i*dx;
-
       /* Extract u at x_i, y_j and four neighboring points */
 
       uij = IJth(udata, i, j);
@@ -263,7 +257,6 @@ static int func(N_Vector u, N_Vector f, void *user_data)
       IJth(fdata, i, j) = hdiff + vdiff + uij - uij*uij*uij + 2.0;
 
     }
-
   }
 
   return(0);
