@@ -2,7 +2,7 @@
  * Programmer(s): Daniel R. Reynolds @ SMU
  *---------------------------------------------------------------
  * LLNS/SMU Copyright Start
- * Copyright (c) 2015, Southern Methodist University and 
+ * Copyright (c) 2017, Southern Methodist University and 
  * Lawrence Livermore National Security
  *
  * This work was performed under the auspices of the U.S. Department 
@@ -15,8 +15,8 @@
  * For details, see the LICENSE file.
  * LLNS/SMU Copyright End
  *---------------------------------------------------------------
- * This is the common header file for the Scaled, Preconditioned
- * Iterative Linear Solvers in ARKODE.
+ * Header file for the Scaled, Preconditioned Iterative Linear 
+ * Solver interface in ARKODE.
  *--------------------------------------------------------------*/
 
 #ifndef _ARKSPILS_H
@@ -24,14 +24,18 @@
 
 #include <sundials/sundials_iterative.h>
 #include <sundials/sundials_nvector.h>
+#include <sundials/sundials_linearsolver.h>
 
 #ifdef __cplusplus  /* wrapper to enable C++ usage */
 extern "C" {
 #endif
 
+/*===============================================================
+ ARKSPILS Constants
+===============================================================*/
 
 /*---------------------------------------------------------------
- ARKSPILS return values 
+ ARKSPILS return values -- ADJUST CONSTANTS AS NECESSARY
 ---------------------------------------------------------------*/
 #define ARKSPILS_SUCCESS       0
 #define ARKSPILS_MEM_NULL     -1
@@ -41,6 +45,7 @@ extern "C" {
 #define ARKSPILS_PMEM_NULL    -5
 #define ARKSPILS_MASSMEM_NULL -6
 
+#define ARKSPILS_SUNLS_FAIL  -10
 
 /*---------------------------------------------------------------
  ARKSPILS solver constants:
@@ -59,12 +64,16 @@ extern "C" {
                    multiplied to get a tolerance on the linear
                    iteration
 ---------------------------------------------------------------*/
-#define ARKSPILS_MAXL   5
-#define ARKSPILS_MSBPRE 50
-#define ARKSPILS_DGMAX  RCONST(0.2)
+#define ARKSPILS_MSBPRE 50 /* -- TURN INTO A PARAMETER, MAKE THIS THE DEFAULT */
+#define ARKSPILS_DGMAX  RCONST(0.2) /* -- TURN INTO A PARAMETER, MAKE THIS THE DEFAULT */
 #define ARKSPILS_EPLIN  RCONST(0.05)
 
 
+  
+/*===============================================================
+ ARKSPILS user-supplied function prototypes
+===============================================================*/
+
 /*---------------------------------------------------------------
  Type: ARKSpilsPrecSetupFn
 
@@ -87,7 +96,7 @@ extern "C" {
  approximation to M.  This function will not be called in
  advance of every call to PrecSolve, but instead will be called
  only as often as necessary to achieve convergence within the
- Newton iteration.  If the PrecSolve function needs no
+ Inexact Newton iteration.  If the PrecSolve function needs no
  preparation, the PrecSetup function can be NULL.
 
  For greater efficiency, the PrecSetup function may save
@@ -113,18 +122,18 @@ extern "C" {
 
  jok     is an input flag indicating whether Jacobian-related
          data needs to be recomputed, as follows:
-           jok == FALSE means recompute Jacobian-related data
+           jok == SUNFALSE means recompute Jacobian-related data
                   from scratch.
-           jok == TRUE  means that Jacobian data, if saved from
+           jok == SUNTRUE  means that Jacobian data, if saved from
                   the previous PrecSetup call, can be reused
                   (with the current value of gamma).
-         A Precset call with jok == TRUE can only occur after
-         a call with jok == FALSE.
+         A Precset call with jok == SUNTRUE can only occur after
+         a call with jok == SUNFALSE.
 
  jcurPtr is a pointer to an output integer flag which is
          to be set by PrecSetup as follows:
-         Set *jcurPtr = TRUE if Jacobian data was recomputed.
-         Set *jcurPtr = FALSE if Jacobian data was not recomputed,
+         Set *jcurPtr = SUNTRUE if Jacobian data was recomputed.
+         Set *jcurPtr = SUNFALSE if Jacobian data was not recomputed,
                         but saved data was reused.
 
  gamma   is the scalar appearing in the Newton matrix.
@@ -132,11 +141,6 @@ extern "C" {
  user_data  is a pointer to user data - the same as the user_data
          parameter passed to the ARKodeSetUserData function.
 
- tmp1, tmp2, and tmp3 are pointers to memory allocated
-                      for N_Vectors which can be used by
-                      ARKSpilsPrecSetupFn as temporary storage or
-                      work space.
-
  NOTE: If the user's preconditioner needs other quantities,
        they are accessible as follows: hcur (the current stepsize)
        and ewt (the error weight vector) are accessible through
@@ -152,11 +156,9 @@ extern "C" {
    < 0 for an unrecoverable error (integration is halted).
 ---------------------------------------------------------------*/
 typedef int (*ARKSpilsPrecSetupFn)(realtype t, N_Vector y, 
-				   N_Vector fy, booleantype jok, 
-				   booleantype *jcurPtr,
-				   realtype gamma, void *user_data,
-				   N_Vector tmp1, N_Vector tmp2,
-				   N_Vector tmp3);
+                                   N_Vector fy, booleantype jok, 
+                                   booleantype *jcurPtr,
+                                   realtype gamma, void *user_data);
 
 
 /*---------------------------------------------------------------
@@ -197,9 +199,6 @@ typedef int (*ARKSpilsPrecSetupFn)(realtype t, N_Vector y,
  user_data  is a pointer to user data - the same as the user_data
          parameter passed to the ARKodeSetUserData function.
 
- tmp    is a pointer to memory allocated for an N_Vector
-        which can be used by PSolve for work space.
-
  Returned value:
  The value to be returned by the PrecSolve function is a flag
  indicating whether it was successful.  This value should be
@@ -208,10 +207,51 @@ typedef int (*ARKSpilsPrecSetupFn)(realtype t, N_Vector y,
    negative for an unrecoverable error (integration is halted).
 ---------------------------------------------------------------*/
 typedef int (*ARKSpilsPrecSolveFn)(realtype t, N_Vector y, 
-				   N_Vector fy, N_Vector r, 
-				   N_Vector z, realtype gamma, 
-				   realtype delta, int lr, 
-				   void *user_data, N_Vector tmp);
+                                   N_Vector fy, N_Vector r, 
+                                   N_Vector z, realtype gamma, 
+                                   realtype delta, int lr, 
+                                   void *user_data);
+
+
+/*---------------------------------------------------------------
+ Type: ARKSpilsJacTimesSetupFn
+
+ The user-supplied Jacobian-times-vector product setup function 
+ JacTimesSetup and the user-supplied Jacobian-times-vector 
+ product function JTimes together must generate the product
+ J*v for v, where J is the Jacobian df/dy, or an approximation 
+ to it, and v is a given vector. It should return 0 if 
+ successful a positive value for a recoverable error or a
+ negative value for an unrecoverable failure.
+
+ Each call to the JacTimesSetup function is preceded by a call 
+ to the RhsFn fi with the same (t,y) arguments.  Thus the 
+ JacTimesSetup function can use any auxiliary data that is 
+ computed and saved by the fi function and made accessible to 
+ JacTimesSetup.
+
+ A function JacTimesSetup must have the prototype given below.
+ Its parameters are as follows:
+
+ t       is the current value of the independent variable.
+
+ y       is the current value of the dependent variable vector,
+          namely the predicted value of y(t).
+
+ fy      is the vector f(t,y).
+
+ user_data  is a pointer to user data - the same as the user_data
+         parameter passed to the ARKodeSetUserData function.
+
+ Returned value:
+ The value to be returned by the JacTimesSetup function is a flag
+ indicating whether it was successful.  This value should be
+   0   if successful,
+   > 0 for a recoverable error (step will be retried),
+   < 0 for an unrecoverable error (integration is halted).
+---------------------------------------------------------------*/
+typedef int (*ARKSpilsJacTimesSetupFn)(realtype t, N_Vector y, 
+                                       N_Vector fy, void *user_data);
 
 
 /*---------------------------------------------------------------
@@ -244,9 +284,38 @@ typedef int (*ARKSpilsPrecSolveFn)(realtype t, N_Vector y,
             which can be used by Jtimes for work space.
 ---------------------------------------------------------------*/
 typedef int (*ARKSpilsJacTimesVecFn)(N_Vector v, N_Vector Jv, 
-				     realtype t, N_Vector y, 
-				     N_Vector fy, void *user_data, 
-				     N_Vector tmp);
+                                     realtype t, N_Vector y, 
+                                     N_Vector fy, void *user_data, 
+                                     N_Vector tmp);
+
+
+/*---------------------------------------------------------------
+ Type: ARKSpilsMassTimesSetupFn
+
+ The user-supplied mass matrix-times-vector product setup 
+ function MassTimesSetup and the user-supplied mass 
+ matrix-times-vector product function MTimes together must 
+ generate the product M*v for v, where M is the mass matrix, or 
+ an approximation to it, and v is a given vector. It should 
+ return 0 if successful a positive value for a recoverable error 
+ or a negative value for an unrecoverable failure.
+
+ A function MassTimesSetup must have the prototype given below.
+ Its parameters are as follows:
+
+ t       is the current value of the independent variable.
+
+ mtimes_data  is a pointer to user data - the same as the 
+         parameter passed to the ARKSpilsSetMassTimes function.
+
+ Returned value:
+ The value to be returned by the MassTimesSetup function is a 
+ flag indicating whether it was successful.  This value should be
+   0   if successful,
+   > 0 for a recoverable error (step will be retried),
+   < 0 for an unrecoverable error (integration is halted).
+---------------------------------------------------------------*/
+typedef int (*ARKSpilsMassTimesSetupFn)(realtype t, void *mtimes_data);
 
 
 /*---------------------------------------------------------------
@@ -266,12 +335,11 @@ typedef int (*ARKSpilsJacTimesVecFn)(N_Vector v, N_Vector Jv,
 
    t        is the current value of the independent variable.
 
-   user_data   is a pointer to user data, the same as the user_data
-            parameter passed to the ARKodeSetUserData function.
+   mtimes_data  is a pointer to user data - the same as the 
+            parameter passed to the ARKSpilsSetMassTimes function.
 ---------------------------------------------------------------*/
 typedef int (*ARKSpilsMassTimesVecFn)(N_Vector v, N_Vector Mv, 
-				      realtype t, void *user_data);
-
+                                      realtype t, void *mtimes_data);
 
 
 /*---------------------------------------------------------------
@@ -298,20 +366,13 @@ typedef int (*ARKSpilsMassTimesVecFn)(N_Vector v, N_Vector Mv,
  user_data  is a pointer to user data - the same as the user_data
          parameter passed to the ARKodeSetUserData function.
 
- tmp1, tmp2, and tmp3 are pointers to memory allocated
-                      for N_Vectors which can be used by
-                      ARKSpilsMassPrecSetupFn as temporary 
-                      storage or work space.
-
  Returned value:
  The value to be returned by the MPrecSetup function is a flag
  indicating whether it was successful.  This value should be
    0   if successful,
    < 0 for an unrecoverable error (integration is halted).
 ---------------------------------------------------------------*/
-typedef int (*ARKSpilsMassPrecSetupFn)(realtype t, void *user_data, 
-				       N_Vector tmp1, N_Vector tmp2, 
-				       N_Vector tmp3);
+typedef int (*ARKSpilsMassPrecSetupFn)(realtype t, void *user_data);
 
 
 /*---------------------------------------------------------------
@@ -346,9 +407,6 @@ typedef int (*ARKSpilsMassPrecSetupFn)(realtype t, void *user_data,
  user_data  is a pointer to user data - the same as the user_data
          parameter passed to the ARKodeSetUserData function.
 
- tmp    is a pointer to memory allocated for an N_Vector
-        which can be used by PSolve for work space.
-
  Returned value:
  The value to be returned by the MPrecSolve function is a flag
  indicating whether it was successful.  This value should be
@@ -356,36 +414,62 @@ typedef int (*ARKSpilsMassPrecSetupFn)(realtype t, void *user_data,
    negative for an unrecoverable error (integration is halted).
 ---------------------------------------------------------------*/
 typedef int (*ARKSpilsMassPrecSolveFn)(realtype t, N_Vector r, 
-				       N_Vector z, realtype delta, 
-				       int lr, void *user_data, 
-				       N_Vector tmp);
-
+                                       N_Vector z, realtype delta, 
+                                       int lr, void *user_data);
+  
+  
+/*===============================================================
+  ARKSPILS Exported functions
+===============================================================*/
 
 /*---------------------------------------------------------------
- Optional inputs to the ARKSPILS linear solver:
+ Required inputs for the ARKSPILS linear solver interface:
 
- ARKSpilsSetPrecType resets the type of preconditioner, pretype,
-                from the value previously set.
-                This must be one of PREC_NONE, PREC_LEFT, 
-                PREC_RIGHT, or PREC_BOTH.
+ ARKSpilsSetLinearSolver specifies the iterative SUNLinearSolver 
+ object that ARKode should use.  This is required if ARKode is 
+ solving an implicit or IMEX IVP, and using an inexact Newton 
+ solver.
 
- ARKSpilsSetGSType specifies the type of Gram-Schmidt
-                orthogonalization to be used. This must be one of
-                the two enumeration constants MODIFIED_GS or
-                CLASSICAL_GS defined in iterative.h. These 
-                correspond to using modified Gram-Schmidt and 
-                classical Gram-Schmidt, respectively.
-                Default value is MODIFIED_GS.
+ ARKSpilsSetMassLinearSolver specifies the iterative 
+ SUNLinearSolver object that ARKode should use for mass-matrix 
+ systems.  This is required if ARKode is solving a problem with 
+ non-identity mass matrix and the user wishes to use an iterative 
+ solver for these systems.
 
- ARKSpilsSetMaxl resets the maximum Krylov subspace size, maxl,
-                from the value previously set.
-                An input value <= 0, gives the default value.
+ NOTE: when solving an implicit or IMEX IVP with non-identity mass
+ matrix and iterative linear solver, both the system and mass solvers
+ must be iterative (i.e. you cannot combine a direct system 
+ solver with an iterative mass matrix solver, etc.).
+
+ The return value is one of:
+    ARKSPILS_SUCCESS   if successful
+    ARKSPILS_MEM_NULL  if the ARKODE memory was NULL
+    ARKSPILS_ILL_INPUT if the linear solver memory was NULL
+---------------------------------------------------------------*/
+SUNDIALS_EXPORT int ARKSpilsSetLinearSolver(void *arkode_mem, 
+                                            SUNLinearSolver LS);
+
+SUNDIALS_EXPORT int ARKSpilsSetMassLinearSolver(void *arkode_mem, 
+                                                SUNLinearSolver LS,
+                                                booleantype time_dep);
+
+  
+/*---------------------------------------------------------------
+ Optional inputs to the ARKSPILS linear solver -- ALL of these 
+ must be called AFTER the corresponding iterative linear solver 
+ object (system matrix or mass matrix) has been attached to the 
+ ARKode integrator).
 
  ARKSpilsSetEpsLin specifies the factor by which the tolerance on
                 the nonlinear iteration is multiplied to get a
                 tolerance on the linear iteration.
                 Default value is 0.05.
 
+ ARKSpilsSetMassEpsLin specifies the factor by which the tolerance
+                on the nonlinear iteration is multiplied to get a
+                tolerance on the mass matrix linear iteration.
+                Default value is 0.05.
+
  ARKSpilsSetPreconditioner specifies the PrecSetup and PrecSolve 
                 functions.  Default is NULL for both arguments 
                 (no preconditioning)
@@ -394,11 +478,15 @@ typedef int (*ARKSpilsMassPrecSolveFn)(realtype t, N_Vector r,
                 and MPrecSolve functions.  Default is NULL for 
                 both arguments (no preconditioning)
 
- ARKSpilsSetJacTimesVecFn specifies the jtimes function. Default 
-                is to use an internal finite difference 
-		approximation routine.
+ ARKSpilsSetJacTimes specifies the jtsetup and jtimes functions. 
+                Default is to use an internal finite difference 
+		approximation routine (no setup).
 
- ARKSpilsSetMassTimesVecFn specifies the mtimes function. No Default.
+ ARKSpilsSetMassTimes specifies the mtsetup and mtimes functions. 
+                Note that there do not exist built-in finite-
+                difference approximation routines for this, this 
+                function MUST be called with non-NULL 'mtimes' 
+                if ARKSpilsSetMassLinearSolver was called.
 
  The return value of ARKSpilsSet* is one of:
     ARKSPILS_SUCCESS      if successful
@@ -407,26 +495,21 @@ typedef int (*ARKSpilsMassPrecSolveFn)(realtype t, N_Vector r,
     ARKSPILS_MASSMEM_NULL if the mass matrix solver memory was NULL
     ARKSPILS_ILL_INPUT    if an input has an illegal value
 ---------------------------------------------------------------*/
-SUNDIALS_EXPORT int ARKSpilsSetPrecType(void *arkode_mem, int pretype);
-SUNDIALS_EXPORT int ARKSpilsSetMassPrecType(void *arkode_mem, int pretype);
-SUNDIALS_EXPORT int ARKSpilsSetGSType(void *arkode_mem, int gstype);
-SUNDIALS_EXPORT int ARKSpilsSetMassGSType(void *arkode_mem, int gstype);
-SUNDIALS_EXPORT int ARKSpilsSetMaxl(void *arkode_mem, int maxl);
-SUNDIALS_EXPORT int ARKSpilsSetMassMaxl(void *arkode_mem, int maxl);
 SUNDIALS_EXPORT int ARKSpilsSetEpsLin(void *arkode_mem, realtype eplifac);
 SUNDIALS_EXPORT int ARKSpilsSetMassEpsLin(void *arkode_mem, realtype eplifac);
 SUNDIALS_EXPORT int ARKSpilsSetPreconditioner(void *arkode_mem, 
-					      ARKSpilsPrecSetupFn pset,
-					      ARKSpilsPrecSolveFn psolve);
+                                              ARKSpilsPrecSetupFn psetup,
+                                              ARKSpilsPrecSolveFn psolve);
 SUNDIALS_EXPORT int ARKSpilsSetMassPreconditioner(void *arkode_mem, 
-						  ARKSpilsMassPrecSetupFn pset,
-						  ARKSpilsMassPrecSolveFn psolve);
-SUNDIALS_EXPORT int ARKSpilsSetJacTimesVecFn(void *arkode_mem, 
-					     ARKSpilsJacTimesVecFn jtv);
-SUNDIALS_EXPORT int ARKSpilsSetMassTimesVecFn(void *arkode_mem, 
-					      ARKSpilsMassTimesVecFn mtv,
-					      void *mtimes_data);
-
+                                                  ARKSpilsMassPrecSetupFn psetup,
+                                                  ARKSpilsMassPrecSolveFn psolve);
+SUNDIALS_EXPORT int ARKSpilsSetJacTimes(void *arkode_mem, 
+                                        ARKSpilsJacTimesSetupFn jtsetup,
+                                        ARKSpilsJacTimesVecFn jtimes);
+SUNDIALS_EXPORT int ARKSpilsSetMassTimes(void *arkode_mem, 
+                                         ARKSpilsMassTimesSetupFn msetup,
+                                         ARKSpilsMassTimesVecFn mtimes,
+                                         void *mtimes_data);
 
 /*---------------------------------------------------------------
  Optional outputs from the ARKSPILS linear solver:
@@ -434,38 +517,22 @@ SUNDIALS_EXPORT int ARKSpilsSetMassTimesVecFn(void *arkode_mem,
  ARKSpilsGetWorkSpace returns the real and integer workspace used
                 by the SPILS module.
 
- ARKSpilsGetMassWorkSpace returns the real and integer workspace used
-                by the mass matrix SPILS module.
-
  ARKSpilsGetNumPrecEvals returns the number of preconditioner
                  evaluations, i.e. the number of calls made
-                 to PrecSetup with jok==FALSE.
-
- ARKSpilsGetNumMassPrecEvals returns the number of mass matrix 
-                 preconditioner evaluations, i.e. the number of 
-                 calls made to MPrecSetup.
+                 to PrecSetup with jok==SUNFALSE.
 
  ARKSpilsGetNumPrecSolves returns the number of calls made to
                  PrecSolve.
 
- ARKSpilsGetNumMassPrecSolves returns the number of calls made to
-                 MPrecSolve.
-
  ARKSpilsGetNumLinIters returns the number of linear iterations.
 
- ARKSpilsGetNumMassIters returns the number of mass matrix solver
-                  iterations.
-
  ARKSpilsGetNumConvFails returns the number of linear
                  convergence failures.
 
- ARKSpilsGetNumMassConvFails returns the number of mass matrix solver
-                 convergence failures.
-
+ ARKSpilsGetNumJTSetupEvals returns the number of calls to jtsetup.
+ 
  ARKSpilsGetNumJtimesEvals returns the number of calls to jtimes.
 
- ARKSpilsGetNumMtimesEvals returns the number of calls to mtimes.
-
  ARKSpilsGetNumRhsEvals returns the number of calls to the user
                  f routine due to finite difference Jacobian
                  times vector evaluation.
@@ -473,6 +540,26 @@ SUNDIALS_EXPORT int ARKSpilsSetMassTimesVecFn(void *arkode_mem,
  ARKSpilsGetLastFlag returns the last error flag set by any of
                  the ARKSPILS interface functions.
 
+ ARKSpilsGetMassWorkSpace returns the real and integer workspace used
+                by the mass matrix SPILS module.
+
+ ARKSpilsGetNumMassPrecEvals returns the number of mass matrix 
+                 preconditioner evaluations, i.e. the number of 
+                 calls made to MPrecSetup.
+
+ ARKSpilsGetNumMassPrecSolves returns the number of calls made to
+                 MPrecSolve.
+
+ ARKSpilsGetNumMassIters returns the number of mass matrix solver
+                  iterations.
+
+ ARKSpilsGetNumMassConvFails returns the number of mass matrix solver
+                 convergence failures.
+
+ ARKSpilsGetNumMTSetups returns the number of calls to mtsetup.
+
+ ARKSpilsGetNumMtimesEvals returns the number of calls to mtimes.
+
  ARKSpilsGetLastMassFlag returns the last error flag set by any of
                  the ARKSPILS interface functions on the mass 
 		 matrix solve.
@@ -483,37 +570,42 @@ SUNDIALS_EXPORT int ARKSpilsSetMassTimesVecFn(void *arkode_mem,
     ARKSPILS_LMEM_NULL if the linear solver memory was NULL
 ---------------------------------------------------------------*/
 SUNDIALS_EXPORT int ARKSpilsGetWorkSpace(void *arkode_mem, 
-					 long int *lenrwLS, 
-					 long int *leniwLS);
-SUNDIALS_EXPORT int ARKSpilsGetMassWorkSpace(void *arkode_mem, 
-					     long int *lenrwMLS, 
-					     long int *leniwMLS);
+                                         long int *lenrwLS, 
+                                         long int *leniwLS);
 SUNDIALS_EXPORT int ARKSpilsGetNumPrecEvals(void *arkode_mem, 
-					    long int *npevals);
-SUNDIALS_EXPORT int ARKSpilsGetNumMassPrecEvals(void *arkode_mem, 
-						long int *nmpevals);
+                                            long int *npevals);
 SUNDIALS_EXPORT int ARKSpilsGetNumPrecSolves(void *arkode_mem, 
-					     long int *npsolves);
-SUNDIALS_EXPORT int ARKSpilsGetNumMassPrecSolves(void *arkode_mem, 
-						 long int *nmpsolves);
+                                             long int *npsolves);
 SUNDIALS_EXPORT int ARKSpilsGetNumLinIters(void *arkode_mem, 
-					   long int *nliters);
-SUNDIALS_EXPORT int ARKSpilsGetNumMassIters(void *arkode_mem, 
-					    long int *nmiters);
+                                           long int *nliters);
 SUNDIALS_EXPORT int ARKSpilsGetNumConvFails(void *arkode_mem, 
-					    long int *nlcfails);
-SUNDIALS_EXPORT int ARKSpilsGetNumMassConvFails(void *arkode_mem, 
-						long int *nmcfails);
+                                            long int *nlcfails);
+SUNDIALS_EXPORT int ARKSpilsGetNumJTSetupEvals(void *arkode_mem,
+                                               long int *njtsetups);
 SUNDIALS_EXPORT int ARKSpilsGetNumJtimesEvals(void *arkode_mem, 
-					      long int *njvevals);
-SUNDIALS_EXPORT int ARKSpilsGetNumMtimesEvals(void *arkode_mem, 
-					      long int *nmvevals);
+                                              long int *njvevals);
 SUNDIALS_EXPORT int ARKSpilsGetNumRhsEvals(void *arkode_mem, 
-					   long int *nfevalsLS); 
+                                           long int *nfevalsLS); 
 SUNDIALS_EXPORT int ARKSpilsGetLastFlag(void *arkode_mem, 
-					long int *flag);
+                                        long int *flag);
+
+SUNDIALS_EXPORT int ARKSpilsGetMassWorkSpace(void *arkode_mem, 
+                                             long int *lenrwMLS, 
+                                             long int *leniwMLS);
+SUNDIALS_EXPORT int ARKSpilsGetNumMassPrecEvals(void *arkode_mem, 
+                                                long int *nmpevals);
+SUNDIALS_EXPORT int ARKSpilsGetNumMassPrecSolves(void *arkode_mem, 
+                                                 long int *nmpsolves);
+SUNDIALS_EXPORT int ARKSpilsGetNumMassIters(void *arkode_mem, 
+                                            long int *nmiters);
+SUNDIALS_EXPORT int ARKSpilsGetNumMassConvFails(void *arkode_mem, 
+                                                long int *nmcfails);
+SUNDIALS_EXPORT int ARKSpilsGetNumMTSetups(void *arkode_mem,
+                                           long int *nmtsetups);
+SUNDIALS_EXPORT int ARKSpilsGetNumMtimesEvals(void *arkode_mem, 
+                                              long int *nmvevals);
 SUNDIALS_EXPORT int ARKSpilsGetLastMassFlag(void *arkode_mem, 
-					    long int *flag);
+                                            long int *flag);
 
 
 /*---------------------------------------------------------------
