@@ -1,7 +1,7 @@
 /*
  * -----------------------------------------------------------------
- * $Revision: 4923 $
- * $Date: 2016-09-19 14:35:51 -0700 (Mon, 19 Sep 2016) $
+ * $Revision$
+ * $Date$
  * ----------------------------------------------------------------- 
  * Programmer(s): Radu Serban @ LLNL
  * -----------------------------------------------------------------
@@ -80,46 +80,46 @@ typedef struct CVodeBMemRec *CVodeBMem;
   
 typedef struct CVodeMemRec {
     
-  realtype cv_uround;         /* machine unit roundoff                        */   
+  realtype cv_uround;         /* machine unit roundoff                         */   
 
   /*-------------------------- 
     Problem Specification Data 
     --------------------------*/
 
-  CVRhsFn cv_f;               /* y' = f(t,y(t))                               */
-  void *cv_user_data;         /* user pointer passed to f                     */
+  CVRhsFn cv_f;               /* y' = f(t,y(t))                                */
+  void *cv_user_data;         /* user pointer passed to f                      */
 
-  int cv_lmm;                 /* lmm = ADAMS or BDF                           */
-  int cv_iter;                /* iter = FUNCTIONAL or NEWTON                  */
+  int cv_lmm;                 /* lmm = ADAMS or BDF                            */
+  int cv_iter;                /* iter = FUNCTIONAL or NEWTON                   */
 
-  int cv_itol;                /* itol = CV_SS, CV_SV, or CV_WF, or CV_NN      */
-  realtype cv_reltol;         /* relative tolerance                           */
-  realtype cv_Sabstol;        /* scalar absolute tolerance                    */
-  N_Vector cv_Vabstol;        /* vector absolute tolerance                    */
-  booleantype cv_user_efun;   /* TRUE if user sets efun                       */
-  CVEwtFn cv_efun;            /* function to set ewt                          */
-  void *cv_e_data;            /* user pointer passed to efun                  */
+  int cv_itol;                /* itol = CV_SS, CV_SV, or CV_WF, or CV_NN       */
+  realtype cv_reltol;         /* relative tolerance                            */
+  realtype cv_Sabstol;        /* scalar absolute tolerance                     */
+  N_Vector cv_Vabstol;        /* vector absolute tolerance                     */
+  booleantype cv_user_efun;   /* SUNTRUE if user sets efun                     */
+  CVEwtFn cv_efun;            /* function to set ewt                           */
+  void *cv_e_data;            /* user pointer passed to efun                   */
 
   /*-----------------------
     Quadrature Related Data 
     -----------------------*/
 
-  booleantype cv_quadr;       /* TRUE if integrating quadratures              */
+  booleantype cv_quadr;       /* SUNTRUE if integrating quadratures            */
 
-  CVQuadRhsFn cv_fQ;          /* q' = fQ(t, y(t))                             */
+  CVQuadRhsFn cv_fQ;          /* q' = fQ(t, y(t))                              */
 
-  booleantype cv_errconQ;     /* TRUE if quadrs. are included in error test   */
+  booleantype cv_errconQ;     /* SUNTRUE if quadrs. are included in error test */
 
-  int cv_itolQ;               /* itolQ = CV_SS or CV_SV                       */
-  realtype cv_reltolQ;        /* relative tolerance for quadratures           */
-  realtype cv_SabstolQ;       /* scalar absolute tolerance for quadratures    */
-  N_Vector cv_VabstolQ;       /* vector absolute tolerance for quadratures    */
+  int cv_itolQ;               /* itolQ = CV_SS or CV_SV                        */
+  realtype cv_reltolQ;        /* relative tolerance for quadratures            */
+  realtype cv_SabstolQ;       /* scalar absolute tolerance for quadratures     */
+  N_Vector cv_VabstolQ;       /* vector absolute tolerance for quadratures     */
 
   /*------------------------
     Sensitivity Related Data 
     ------------------------*/
 
-  booleantype cv_sensi;       /* TRUE if computing sensitivities              */
+  booleantype cv_sensi;       /* SUNTRUE if computing sensitivities           */
 
   int cv_Ns;                  /* Number of sensitivities                      */
 
@@ -128,7 +128,7 @@ typedef struct CVodeMemRec {
   CVSensRhsFn cv_fS;          /* fS = (df/dy)*yS + (df/dp)                    */
   CVSensRhs1Fn cv_fS1;        /* fS1 = (df/dy)*yS_i + (df/dp)                 */
   void *cv_fS_data;           /* data pointer passed to fS                    */
-  booleantype cv_fSDQ;        /* TRUE if using internal DQ functions          */
+  booleantype cv_fSDQ;        /* SUNTRUE if using internal DQ functions       */
   int cv_ifS;                 /* ifS = ALLSENS or ONESENS                     */
 
   realtype *cv_p;             /* parameters in f(t,y,p)                       */
@@ -137,7 +137,7 @@ typedef struct CVodeMemRec {
   int cv_DQtype;              /* central/forward finite differences           */
   realtype cv_DQrhomax;       /* cut-off value for separate/simultaneous FD   */
 
-  booleantype cv_errconS;     /* TRUE if yS are considered in err. control    */
+  booleantype cv_errconS;     /* SUNTRUE if yS are considered in err. control */
 
   int cv_itolS;
   realtype cv_reltolS;        /* relative tolerance for sensitivities         */
@@ -148,13 +148,13 @@ typedef struct CVodeMemRec {
     Quadrature Sensitivity Related Data 
     -----------------------------------*/
 
-  booleantype cv_quadr_sensi; /* TRUE if computing sensitivties of quadrs.    */
+  booleantype cv_quadr_sensi; /* SUNTRUE if computing sensitivties of quadrs. */
 
   CVQuadSensRhsFn cv_fQS;     /* fQS = (dfQ/dy)*yS + (dfQ/dp)                 */
   void *cv_fQS_data;          /* data pointer passed to fQS                   */
-  booleantype cv_fQSDQ;       /* TRUE if using internal DQ functions          */
+  booleantype cv_fQSDQ;       /* SUNTRUE if using internal DQ functions       */
 
-  booleantype cv_errconQS;    /* TRUE if yQS are considered in err. con.      */
+  booleantype cv_errconQS;    /* SUNTRUE if yQS are considered in err. con.   */
 
   int cv_itolQS;
   realtype cv_reltolQS;       /* relative tolerance for yQS                   */
@@ -283,7 +283,7 @@ typedef struct CVodeMemRec {
 			      user that t + h == t for the next internal step */
   int cv_maxnef;           /* maximum number of error test failures           */
   int cv_maxncf;           /* maximum number of nonlinear conv. failures      */
-
+  
   realtype cv_hmin;        /* |h| >= hmin                                     */
   realtype cv_hmax_inv;    /* |h| <= 1/hmax_inv                               */
   realtype cv_etamax;      /* eta <= etamax                                   */
@@ -325,12 +325,12 @@ typedef struct CVodeMemRec {
     Space requirements for CVODES 
     -----------------------------*/
 
-  long int cv_lrw1;        /* no. of realtype words in 1 N_Vector y           */ 
-  long int cv_liw1;        /* no. of integer words in 1 N_Vector y            */ 
-  long int cv_lrw1Q;       /* no. of realtype words in 1 N_Vector yQ          */ 
-  long int cv_liw1Q;       /* no. of integer words in 1 N_Vector yQ           */ 
-  long int cv_lrw;         /* no. of realtype words in CVODES work vectors    */
-  long int cv_liw;         /* no. of integer words in CVODES work vectors     */
+  sunindextype cv_lrw1;        /* no. of realtype words in 1 N_Vector y           */ 
+  sunindextype cv_liw1;        /* no. of integer words in 1 N_Vector y            */ 
+  sunindextype cv_lrw1Q;       /* no. of realtype words in 1 N_Vector yQ          */ 
+  sunindextype cv_liw1Q;       /* no. of integer words in 1 N_Vector yQ           */ 
+  long int cv_lrw;             /* no. of realtype words in CVODES work vectors    */
+  long int cv_liw;             /* no. of integer words in CVODES work vectors     */
 
   /*----------------
     Step size ratios
@@ -375,13 +375,13 @@ typedef struct CVodeMemRec {
   realtype cv_hu;              /* last successful h value used                */
   realtype cv_saved_tq5;       /* saved value of tq[5]                        */
   booleantype cv_jcur;         /* is Jacobian info for linear solver current? */
+  int cv_convfail;             /* flag storing previous solver failure mode   */
   realtype cv_tolsf;           /* tolerance scale factor                      */
   int cv_qmax_alloc;           /* qmax used when allocating mem               */
   int cv_qmax_allocQ;          /* qmax used when allocating quad. mem         */
   int cv_qmax_allocS;          /* qmax used when allocating sensi. mem        */
   int cv_qmax_allocQS;         /* qmax used when allocating quad. sensi. mem  */
   int cv_indx_acor;            /* index of zn vector in which acor is saved   */
-  booleantype cv_setupNonNull; /* Does setup do something?                    */
 
   /*--------------------------------------------------------------------
     Flags turned ON by CVodeInit, CVodeSensMalloc, and CVodeQuadMalloc 
@@ -445,7 +445,7 @@ typedef struct CVodeMemRec {
     Adjoint sensitivity data
     ------------------------*/
 
-  booleantype cv_adj;             /* TRUE if performing ASA                   */
+  booleantype cv_adj;             /* SUNTRUE if performing ASA                */
 
   struct CVadjMemRec *cv_adj_mem; /* Pointer to adjoint memory structure      */
 
@@ -825,12 +825,12 @@ struct CVadjMemRec {
  * fpred - f(tn, ypred).
  *
  * jcurPtr - a pointer to a boolean to be filled in by cv_lsetup.
- *           The function should set *jcurPtr=TRUE if its Jacobian
+ *           The function should set *jcurPtr=SUNTRUE if its Jacobian
  *           data is current after the call and should set
- *           *jcurPtr=FALSE if its Jacobian data is not current.
+ *           *jcurPtr=SUNFALSE if its Jacobian data is not current.
  *           Note: If cv_lsetup calls for re-evaluation of
  *           Jacobian data (based on convfail and CVODE state
- *           data), it should return *jcurPtr=TRUE always;
+ *           data), it should return *jcurPtr=SUNTRUE always;
  *           otherwise an infinite loop can result.
  *
  * vtemp1 - temporary N_Vector provided for use by cv_lsetup.
