@@ -1,8 +1,4 @@
-/*
- * -----------------------------------------------------------------
- * $Revision: 4869 $
- * $Date: 2016-08-19 10:34:20 -0700 (Fri, 19 Aug 2016) $
- * ----------------------------------------------------------------- 
+/* ----------------------------------------------------------------- 
  * Programmer(s): David J. Gardner and Carol S. Woodward @ LLNL
  * -----------------------------------------------------------------
  * Acknowledgements: This NVECTOR module is based on the NVECTOR 
@@ -22,8 +18,7 @@
  * -----------------------------------------------------------------
  * This is the implementation file for an OpenMP implementation
  * of the NVECTOR module.
- * -----------------------------------------------------------------
- */
+ * -----------------------------------------------------------------*/
 
 #include <omp.h>
 
@@ -79,7 +74,7 @@ N_Vector_ID N_VGetVectorID_OpenMP(N_Vector v)
  * Function to create a new empty vector 
  */
 
-N_Vector N_VNewEmpty_OpenMP(long int length, int num_threads)
+N_Vector N_VNewEmpty_OpenMP(sunindextype length, int num_threads)
 {
   N_Vector v;
   N_Vector_Ops ops;
@@ -129,7 +124,7 @@ N_Vector N_VNewEmpty_OpenMP(long int length, int num_threads)
 
   content->length   = length;
   content->num_threads = num_threads;
-  content->own_data = FALSE;
+  content->own_data = SUNFALSE;
   content->data     = NULL;
 
   /* Attach content and ops */
@@ -143,7 +138,7 @@ N_Vector N_VNewEmpty_OpenMP(long int length, int num_threads)
  * Function to create a new vector 
  */
 
-N_Vector N_VNew_OpenMP(long int length, int num_threads)
+N_Vector N_VNew_OpenMP(sunindextype length, int num_threads)
 {
   N_Vector v;
   realtype *data;
@@ -161,7 +156,7 @@ N_Vector N_VNew_OpenMP(long int length, int num_threads)
     if(data == NULL) { N_VDestroy_OpenMP(v); return(NULL); }
 
     /* Attach data */
-    NV_OWN_DATA_OMP(v) = TRUE;
+    NV_OWN_DATA_OMP(v) = SUNTRUE;
     NV_DATA_OMP(v)     = data;
 
   }
@@ -173,7 +168,7 @@ N_Vector N_VNew_OpenMP(long int length, int num_threads)
  * Function to create a vector with user data component 
  */
 
-N_Vector N_VMake_OpenMP(long int length, realtype *v_data, int num_threads)
+N_Vector N_VMake_OpenMP(sunindextype length, realtype *v_data, int num_threads)
 {
   N_Vector v;
 
@@ -183,7 +178,7 @@ N_Vector N_VMake_OpenMP(long int length, realtype *v_data, int num_threads)
 
   if (length > 0) {
     /* Attach data */
-    NV_OWN_DATA_OMP(v) = FALSE;
+    NV_OWN_DATA_OMP(v) = SUNFALSE;
     NV_DATA_OMP(v)     = v_data;
   }
 
@@ -262,19 +257,27 @@ void N_VDestroyVectorArray_OpenMP(N_Vector *vs, int count)
 /* ----------------------------------------------------------------------------
  * Function to return number of vector elements
  */
-long int N_VGetLength_OpenMP(N_Vector v)
+sunindextype N_VGetLength_OpenMP(N_Vector v)
 {
   return NV_LENGTH_OMP(v);
 }
 
- 
 /* ----------------------------------------------------------------------------
- * Function to print a vector 
+ * Function to print a vector to stdout
  */
  
 void N_VPrint_OpenMP(N_Vector x)
 {
-  long int i, N;
+  N_VPrintFile_OpenMP(x, stdout);
+}
+ 
+/* ----------------------------------------------------------------------------
+ * Function to print a vector to outfile
+ */
+ 
+void N_VPrintFile_OpenMP(N_Vector x, FILE *outfile)
+{
+  sunindextype i, N;
   realtype *xd;
 
   xd = NULL;
@@ -284,14 +287,14 @@ void N_VPrint_OpenMP(N_Vector x)
 
   for (i = 0; i < N; i++) {
 #if defined(SUNDIALS_EXTENDED_PRECISION)
-    printf("%11.8Lg\n", xd[i]);
+    fprintf(outfile, "%11.8Lg\n", xd[i]);
 #elif defined(SUNDIALS_DOUBLE_PRECISION)
-    printf("%11.8g\n", xd[i]);
+    fprintf(outfile, "%11.8g\n", xd[i]);
 #else
-    printf("%11.8g\n", xd[i]);
+    fprintf(outfile, "%11.8g\n", xd[i]);
 #endif
   }
-  printf("\n");
+  fprintf(outfile, "\n");
 
   return;
 }
@@ -358,7 +361,7 @@ N_Vector N_VCloneEmpty_OpenMP(N_Vector w)
 
   content->length   = NV_LENGTH_OMP(w);
   content->num_threads   = NV_NUM_THREADS_OMP(w);
-  content->own_data = FALSE;
+  content->own_data = SUNFALSE;
   content->data     = NULL;
 
   /* Attach content and ops */
@@ -377,7 +380,7 @@ N_Vector N_VClone_OpenMP(N_Vector w)
 {
   N_Vector v;
   realtype *data;
-  long int length;
+  sunindextype length;
 
   v = NULL;
   v = N_VCloneEmpty_OpenMP(w);
@@ -394,7 +397,7 @@ N_Vector N_VClone_OpenMP(N_Vector w)
     if(data == NULL) { N_VDestroy_OpenMP(v); return(NULL); }
 
     /* Attach data */
-    NV_OWN_DATA_OMP(v) = TRUE;
+    NV_OWN_DATA_OMP(v) = SUNTRUE;
     NV_DATA_OMP(v)     = data;
 
   }
@@ -409,7 +412,7 @@ N_Vector N_VClone_OpenMP(N_Vector w)
 
 void N_VDestroy_OpenMP(N_Vector v)
 {
-  if (NV_OWN_DATA_OMP(v) == TRUE) {
+  if (NV_OWN_DATA_OMP(v) == SUNTRUE) {
     free(NV_DATA_OMP(v));
     NV_DATA_OMP(v) = NULL;
   }
@@ -425,7 +428,7 @@ void N_VDestroy_OpenMP(N_Vector v)
  * Get storage requirement for N_Vector
  */
 
-void N_VSpace_OpenMP(N_Vector v, long int *lrw, long int *liw)
+void N_VSpace_OpenMP(N_Vector v, sunindextype *lrw, sunindextype *liw)
 {
   *lrw = NV_LENGTH_OMP(v);
   *liw = 1;
@@ -462,7 +465,7 @@ void N_VSetArrayPointer_OpenMP(realtype *v_data, N_Vector v)
 
 void N_VLinearSum_OpenMP(realtype a, N_Vector x, realtype b, N_Vector y, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype c, *xd, *yd, *zd;
   N_Vector v1, v2;
   booleantype test;
@@ -556,7 +559,7 @@ void N_VLinearSum_OpenMP(realtype a, N_Vector x, realtype b, N_Vector y, N_Vecto
 
 void N_VConst_OpenMP(realtype c, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *zd;
 
   zd = NULL;
@@ -578,7 +581,7 @@ void N_VConst_OpenMP(realtype c, N_Vector z)
 
 void N_VProd_OpenMP(N_Vector x, N_Vector y, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *yd, *zd;
 
   xd = yd = zd = NULL;
@@ -603,7 +606,7 @@ void N_VProd_OpenMP(N_Vector x, N_Vector y, N_Vector z)
 
 void N_VDiv_OpenMP(N_Vector x, N_Vector y, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *yd, *zd;
 
   xd = yd = zd = NULL;
@@ -628,7 +631,7 @@ void N_VDiv_OpenMP(N_Vector x, N_Vector y, N_Vector z)
 
 void N_VScale_OpenMP(realtype c, N_Vector x, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *zd;
 
   xd = zd = NULL;
@@ -663,7 +666,7 @@ void N_VScale_OpenMP(realtype c, N_Vector x, N_Vector z)
 
 void N_VAbs_OpenMP(N_Vector x, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *zd;
 
   xd = zd = NULL;
@@ -686,7 +689,7 @@ void N_VAbs_OpenMP(N_Vector x, N_Vector z)
 
 void N_VInv_OpenMP(N_Vector x, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *zd;
 
   xd = zd = NULL;
@@ -710,7 +713,7 @@ void N_VInv_OpenMP(N_Vector x, N_Vector z)
 
 void N_VAddConst_OpenMP(N_Vector x, realtype b, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *zd;
 
   xd = zd = NULL;
@@ -734,7 +737,7 @@ void N_VAddConst_OpenMP(N_Vector x, realtype b, N_Vector z)
 
 realtype N_VDotProd_OpenMP(N_Vector x, N_Vector y)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype sum, *xd, *yd;
 
   sum = ZERO;
@@ -760,7 +763,7 @@ realtype N_VDotProd_OpenMP(N_Vector x, N_Vector y)
 
 realtype N_VMaxNorm_OpenMP(N_Vector x)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype tmax, max, *xd;
 
   max = ZERO;
@@ -793,7 +796,7 @@ realtype N_VMaxNorm_OpenMP(N_Vector x)
 
 realtype N_VWrmsNorm_OpenMP(N_Vector x, N_Vector w)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype sum, *xd, *wd;
 
   sum = ZERO;
@@ -819,7 +822,7 @@ realtype N_VWrmsNorm_OpenMP(N_Vector x, N_Vector w)
 
 realtype N_VWrmsNormMask_OpenMP(N_Vector x, N_Vector w, N_Vector id)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype sum, *xd, *wd, *idd;
 
   sum = ZERO;
@@ -848,7 +851,7 @@ realtype N_VWrmsNormMask_OpenMP(N_Vector x, N_Vector w, N_Vector id)
 
 realtype N_VMin_OpenMP(N_Vector x)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype min, *xd;
   realtype tmin;
 
@@ -885,7 +888,7 @@ realtype N_VMin_OpenMP(N_Vector x)
 
 realtype N_VWL2Norm_OpenMP(N_Vector x, N_Vector w)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype sum, *xd, *wd;
 
   sum = ZERO;
@@ -911,7 +914,7 @@ realtype N_VWL2Norm_OpenMP(N_Vector x, N_Vector w)
 
 realtype N_VL1Norm_OpenMP(N_Vector x)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype sum, *xd;
 
   sum = ZERO;
@@ -935,7 +938,7 @@ realtype N_VL1Norm_OpenMP(N_Vector x)
 
 void N_VCompare_OpenMP(realtype c, N_Vector x, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *zd;
 
   xd = zd = NULL;
@@ -960,7 +963,7 @@ void N_VCompare_OpenMP(realtype c, N_Vector x, N_Vector z)
 
 booleantype N_VInvTest_OpenMP(N_Vector x, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *zd, val;
 
   xd = zd = NULL;
@@ -981,9 +984,9 @@ booleantype N_VInvTest_OpenMP(N_Vector x, N_Vector z)
   }
 
   if (val > ZERO)
-    return (FALSE);
+    return (SUNFALSE);
   else
-    return (TRUE);
+    return (SUNTRUE);
 }
 
 
@@ -993,7 +996,7 @@ booleantype N_VInvTest_OpenMP(N_Vector x, N_Vector z)
 
 booleantype N_VConstrMask_OpenMP(N_Vector c, N_Vector x, N_Vector m)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype temp;
   realtype *cd, *xd, *md;
 
@@ -1020,8 +1023,8 @@ booleantype N_VConstrMask_OpenMP(N_Vector c, N_Vector x, N_Vector m)
     }
   }
 
-  if (temp == ONE) return (TRUE);
-  else return(FALSE);
+  if (temp == ONE) return (SUNTRUE);
+  else return(SUNFALSE);
 }
 
 
@@ -1031,7 +1034,7 @@ booleantype N_VConstrMask_OpenMP(N_Vector c, N_Vector x, N_Vector m)
 
 realtype N_VMinQuotient_OpenMP(N_Vector num, N_Vector denom)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *nd, *dd, min, tmin, val;
 
   nd = dd = NULL;
@@ -1077,7 +1080,7 @@ realtype N_VMinQuotient_OpenMP(N_Vector num, N_Vector denom)
 
 static void VCopy_OpenMP(N_Vector x, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *zd;
 
   xd = zd = NULL;
@@ -1101,7 +1104,7 @@ static void VCopy_OpenMP(N_Vector x, N_Vector z)
 
 static void VSum_OpenMP(N_Vector x, N_Vector y, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *yd, *zd;
 
   xd = yd = zd = NULL;
@@ -1126,7 +1129,7 @@ static void VSum_OpenMP(N_Vector x, N_Vector y, N_Vector z)
 
 static void VDiff_OpenMP(N_Vector x, N_Vector y, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *yd, *zd;
 
   xd = yd = zd = NULL;
@@ -1151,7 +1154,7 @@ static void VDiff_OpenMP(N_Vector x, N_Vector y, N_Vector z)
 
 static void VNeg_OpenMP(N_Vector x, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *zd;
 
   xd = zd = NULL;
@@ -1175,7 +1178,7 @@ static void VNeg_OpenMP(N_Vector x, N_Vector z)
 
 static void VScaleSum_OpenMP(realtype c, N_Vector x, N_Vector y, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *yd, *zd;
 
   xd = yd = zd = NULL;
@@ -1200,7 +1203,7 @@ static void VScaleSum_OpenMP(realtype c, N_Vector x, N_Vector y, N_Vector z)
 
 static void VScaleDiff_OpenMP(realtype c, N_Vector x, N_Vector y, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *yd, *zd;
 
   xd = yd = zd = NULL;
@@ -1225,7 +1228,7 @@ static void VScaleDiff_OpenMP(realtype c, N_Vector x, N_Vector y, N_Vector z)
 
 static void VLin1_OpenMP(realtype a, N_Vector x, N_Vector y, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *yd, *zd;
 
   xd = yd = zd = NULL;
@@ -1250,7 +1253,7 @@ static void VLin1_OpenMP(realtype a, N_Vector x, N_Vector y, N_Vector z)
 
 static void VLin2_OpenMP(realtype a, N_Vector x, N_Vector y, N_Vector z)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *yd, *zd;
 
   xd = yd = zd = NULL;
@@ -1275,7 +1278,7 @@ static void VLin2_OpenMP(realtype a, N_Vector x, N_Vector y, N_Vector z)
 
 static void Vaxpy_OpenMP(realtype a, N_Vector x, N_Vector y)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd, *yd;
 
   xd = yd = NULL;
@@ -1315,7 +1318,7 @@ static void Vaxpy_OpenMP(realtype a, N_Vector x, N_Vector y)
 
 static void VScaleBy_OpenMP(realtype a, N_Vector x)
 {
-  long int i, N;
+  sunindextype i, N;
   realtype *xd;
 
   xd = NULL;
